/**
 * Maa ewolucja - Gra ewolucyjno spoeczna
 *
 *	Klasa odpowiadajca za zarzdzani ustawieniami programu.
 *
 * @author Adam Makowski
 */

package settings;

import java.awt.Color;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.Scanner;

public class Settings extends FileManager {
	
	
	/**
	 * Dostpne kolory w palecie.
	 * @author amalkowski
	 */
	public enum PaletteColors {
		COLOR_MINUS_2, COLOR_MINUS_1, COLOR_0, COLOR_PLUS_1, COLOR_PLUS_2, COLOR_A, COLOR_B, COLOR_C, COLOR_D
	};

	/**
	 * Pliki z ustawieniami.
	 */
	public static final String DIR_TO_COLOR_FILE = "./color/UserInterfaceColours.txt";
	public static final String DIR_TO_COLOR_DIRECTORY = "./color";
	public static final String SETTING_FILE_NAME = "settings.txt";
	/**
	 * Zbir par kod koloru - color.
	 */
	private static HashMap<String, Color> colors = new HashMap<String, Color>();
	/**
	 * Aktualnie uywana paleta kolorw.
	 */
	private static HashMap<PaletteColors, Color> palette = new HashMap<PaletteColors, Color>();
	/**
	 * Czy w programie maj by tworzone logi?
	 */
	private static boolean log = false;

	/**
	 * Wczytaj parametry z pliku.
	 */
	public static void loadParams()
	{
		String tmp;
		File file;
		file = new File(SETTING_FILE_NAME);
		if(!file.exists())
			return;

	    Scanner in = null;
		try
		{
			in = new Scanner(file);
			while(in.hasNextLine())
			{
				tmp=in.nextLine();
				if(tmp.startsWith("log"))
				{
					tmp = tmp.substring(6);
					if(tmp.replace(" ", "").equals("yes") || tmp.replace(" ", "").equals("YES"))
						log = true;
				}
			}
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * Czy w programie tworzone bd logi?
	 */
	public static boolean isLog() {
		return log;
	}

	/**
	 * Wczytaj palet kolorw z pliku.
	 */
	public static void loadPalette() {
		String paletteName = findPaletteName();
		File paletteFile = null, directory;
		directory = new File(DIR_TO_COLOR_DIRECTORY);
		String[] palletes = directory.list();

		for (String tmp : palletes)
			if (tmp.startsWith("palette_" + paletteName + ".txt")) {
				paletteFile = new File(DIR_TO_COLOR_DIRECTORY + "/" + tmp);
				break;
			}

		if (!directory.exists() || paletteFile == null || paletteName.equals("")) {
			makeDefaultPalette();
			return;
		}
		Scanner in = null;
		try {
			in = new Scanner(paletteFile);
			fillPalette(in);

		} catch (FileNotFoundException e) {
			e.printStackTrace();

		} finally {
			in.close();
		}
	}

	/**
	 * Zwr kolor z palety
	 * @param color Kod koloru
	 */
	private static Color getColorFromPallette(String color)
	{
		switch(color)
		{
		case "COLOR_MINUS_2":
			return palette.get(PaletteColors.COLOR_MINUS_2);
		case "COLOR_MINUS_1":
			return palette.get(PaletteColors.COLOR_MINUS_1);
		case "COLOR_0":
			return palette.get(PaletteColors.COLOR_0);
		case "COLOR_PLUS_1":
			return palette.get(PaletteColors.COLOR_PLUS_1);
		case "COLOR_PLUS_2":
			return palette.get(PaletteColors.COLOR_PLUS_2);
		case "COLOR_A":
			return palette.get(PaletteColors.COLOR_A);
		case "COLOR_B":
			return palette.get(PaletteColors.COLOR_B);
		case "COLOR_C":
			return palette.get(PaletteColors.COLOR_C);
		case "COLOR_D":
			return palette.get(PaletteColors.COLOR_D);
		}
		return null;
	}

	/**
	 * Wypenij palet kolorami.
	 */
	private static void fillPalette(Scanner in) {
		String record;
		while (in.hasNextLine()) {
			record = in.nextLine();
			if (record.startsWith("COLOR_MINUS_2 "))
				palette.put(PaletteColors.COLOR_MINUS_2, parseColor(record.substring("COLOR_MINUS_2 ".length()+1)));
			else if (record.startsWith("COLOR_MINUS_1 "))
				palette.put(PaletteColors.COLOR_MINUS_1, parseColor(record.substring("COLOR_MINUS_1 ".length()+1)));
			else if (record.startsWith("COLOR_0 "))
				palette.put(PaletteColors.COLOR_0, parseColor(record.substring("COLOR_0 ".length()+1)));
			else if (record.startsWith("COLOR_PLUS_1 "))
				palette.put(PaletteColors.COLOR_PLUS_1, parseColor(record.substring("COLOR_PLUS_1 ".length()+1)));
			else if (record.startsWith("COLOR_PLUS_2 "))
				palette.put(PaletteColors.COLOR_PLUS_2, parseColor(record.substring("COLOR_PLUS_2 ".length()+1)));
			else if (record.startsWith("COLOR_A "))
				palette.put(PaletteColors.COLOR_A, parseColor(record.substring("COLOR_A ".length()+1)));
			else if (record.startsWith("COLOR_B "))
				palette.put(PaletteColors.COLOR_B, parseColor(record.substring("COLOR_B ".length()+1)));
			else if (record.startsWith("COLOR_C "))
				palette.put(PaletteColors.COLOR_C, parseColor(record.substring("COLOR_C ".length()+1)));
			else if (record.startsWith("COLOR_D "))
				palette.put(PaletteColors.COLOR_D, parseColor(record.substring("COLOR_D ".length()+1)));
		}
	}

	/**
	 * Znajdz nazw palety.
	 */
	private static String findPaletteName() {
		String record;
		File colors;

		colors = new File(DIR_TO_COLOR_FILE);
		if (!colors.exists()) {
			return "";
		}
		Scanner in = null;
		try {
			in = new Scanner(colors);

			while (in.hasNextLine()) {
				record = in.nextLine();
				if (record.startsWith("##PALETTE ")) {
					return record.substring("##PALETTE ".length());
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();

		} finally {
			in.close();
		}
		return "";
	}

	/**
	 * Dodaj domylne kolory programu.
	 */
	private static void makeDefaultPalette() {
		palette.put(PaletteColors.COLOR_MINUS_2, new Color(1, 0, 0));
		palette.put(PaletteColors.COLOR_MINUS_1, new Color(2, 0, 0));
		palette.put(PaletteColors.COLOR_0, new Color(3, 0, 0));
		palette.put(PaletteColors.COLOR_PLUS_1, new Color(4, 0, 0));
		palette.put(PaletteColors.COLOR_PLUS_2, new Color(5, 0, 0));
		palette.put(PaletteColors.COLOR_A, new Color(6, 0, 0));
		palette.put(PaletteColors.COLOR_B, new Color(7, 0, 0));
		palette.put(PaletteColors.COLOR_C, new Color(8, 0, 0));
		palette.put(PaletteColors.COLOR_D, new Color(9, 0, 0));
	}

	/**
	 * Wczytaj kolory z pliku.
	 */
	public static void loadColors() {
		File colors;
		colors = new File(DIR_TO_COLOR_FILE);
		if (!colors.exists()) {
			return;
		}
		Scanner in = null;
		try {
			in = new Scanner(colors);
			fillColors(in);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} finally {
			in.close();
		}
	}

	/**
	 * Stwrz obiekt klasy Color na podstawie acuchu rgb.
	 */
	private static Color parseColor(String color) {

		int r = Integer.parseInt(color.substring(0, 2), 16);
		int g = Integer.parseInt(color.substring(2, 4), 16);
		int b = Integer.parseInt(color.substring(4, 6), 16);

		if (!(r >= 0 && r <= 255 && g >= 0 && g <= 255 && b >= 0 && b <= 255))
			return Color.WHITE;
		else
			return new Color(r, g, b);
	}

	/**
	 * Wypenij pary kod koloru - kolor.
	 */
	private static void fillColors(Scanner in) {
		String record, code;
		String text;
		Integer indexOfSpace;
		while (in.hasNextLine()) {
			record = in.nextLine();

			if (isComment(record))
				continue;

			if (record.startsWith("##PALETTE "))
				continue;

			indexOfSpace = record.indexOf(" ");

			if (indexOfSpace == -1)
				continue;

			code = record.substring(0, indexOfSpace);
			text = record.substring(indexOfSpace + 1);

			if(text.startsWith("#"))
				colors.put(code, parseColor(text.substring(1)));
			else if(text.startsWith("COLOR_"))
			{
				colors.put(code, getColorFromPallette(text));
			}
		}
	}

	/**
	 * Zwr kolor na podstawie podane kodu. Domylnie, jeeli nie ma takiego kodu, szary.
	 * @param code Kod koloru
	 */
	public static Color getColor(String code) {
		Color tmp = colors.get(code);
		if (tmp == null) {
			tmp = Color.GRAY;
		}
		return tmp;
	}
}
