/**
 * Maa ewolucja - Gra ewolucyjno spoeczna
 *
 *	Klasa reprezentujca wykres.
 *
 * @author Grzegorz Przytua, Adam Makowski
 */

package view;

import java.awt.Color;
import java.awt.Font;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.entity.ChartEntity;
import org.jfree.chart.entity.XYItemEntity;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.Range;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import settings.Dictionary;
import settings.Settings;

@SuppressWarnings("serial")
public class Chart extends JPanel
{
	/**
	 * Pozycje na wykresie wartoci rednich.
	 */
	private XYSeries thiefSeries;
	private XYSeries producerSeries;
	private XYSeries mutatedThiefSeries;
	private XYSeries mutatedProducerSeries;

	/**
	 * Pozycje na wykresie wartoci chwilowych.
	 */
	private XYSeries normalSeries;
	private XYSeries mutatedSeries;

	/**
	 * Wykres i jego dane.
	 */
	private JFreeChart theChart;
	private final XYSeriesCollection dataset;
	
	/**
	 * Obrazki wywietlane przy wykresie wartoci chwilowych.
	 */
	private JLabel image1 = new JLabel();
	private JLabel image2 = new JLabel();
	private JLabel image3 = new JLabel();

	private final ChartPanel chartPanel;

	/**
	 * Odniesienie do panelu prezentujcego struktur populacji w trakcie cyklu.
	 */
	private PopulationProportionPane proportionPane;

	/**
	 * Konstruktor wykresu
	 * @param thiefSeriesName
	 * @param producerSeriesName
	 * @param mutatedThiefSeriesName
	 * @param mutatedProducerSeriesName
	 * @param populationAmount
	 * @param title
	 */
	public Chart(String thiefSeriesName, String producerSeriesName, String mutatedThiefSeriesName,
			String mutatedProducerSeriesName,int populationAmount, String title)
	{
		this.thiefSeries = new XYSeries(thiefSeriesName, true, true);
		this.producerSeries = new XYSeries(producerSeriesName, true, true);
		this.mutatedThiefSeries = new XYSeries(mutatedThiefSeriesName, true, true);
		this.mutatedProducerSeries = new XYSeries(mutatedProducerSeriesName, true, true);

		dataset = new XYSeriesCollection(this.thiefSeries);
		dataset.addSeries(producerSeries);
		dataset.addSeries(mutatedThiefSeries);
		dataset.addSeries(mutatedProducerSeries);

		theChart = createChart(dataset);
		chartPanel = new ChartPanel(theChart);
		addMouseListener();
		setLayout(null);

		add(chartPanel);

		chartPanel.setLocation(25, 0);
		chartPanel.setSize(600, 440);

		XYPlot plot = (XYPlot) theChart.getPlot();

		setColors(theChart);
		setTitle(title, theChart);

		plot.getRenderer().setSeriesPaint(0,Settings.getColor("THIEF_COLOR_SIMULATION"));
		plot.getRenderer().setSeriesPaint(1,Settings.getColor("PRODUCER_COLOR_SIMULATION"));
		plot.getRenderer().setSeriesPaint(2,Settings.getColor("MUTATED_THIEF_COLOR_SIMULATION"));
		plot.getRenderer().setSeriesPaint(3,Settings.getColor("MUTATED_PRODUCER_COLOR_SIMULATION"));
		plot.setNoDataMessage(Dictionary.getText("MEAN_CHART_MESSAGE"));
		plot.setNoDataMessageFont(new Font("Tahoma", Font.BOLD, 40));
		plot.setNoDataMessagePaint(Settings.getColor("SIMULATION_WINDOW_FONT"));
		theChart.getXYPlot().getRangeAxis().setRange(new Range(-populationAmount/20,1.05*populationAmount));
	}

	/**
	 * Dodanie zalenoci pomidzy wykresami a panelem reprezentujcym struktur populacji.
	 */
	private void addMouseListener() {
		chartPanel.addChartMouseListener(new ChartMouseListener() {

			@Override
			public void chartMouseMoved(ChartMouseEvent event) {
				for(int i = 0; i < dataset.getSeriesCount(); i++)
					if(((XYSeries) dataset.getSeries().get(i)).isEmpty())
						return;
				ChartEntity entity = event.getEntity();
				if (entity == null)
					return;
				if(!(entity instanceof XYItemEntity))
					return;
				int item = ((XYItemEntity) entity).getItem();
				proportionPane.markCycle(item);
			}
			@Override
			public void chartMouseClicked(ChartMouseEvent event) {
				proportionPane.markCycle(-1);
			}
		});
	}

	private void setTitle(String title, JFreeChart chart)
	{
		theChart.setTitle(title);
		theChart.getTitle().setFont(new Font("Tahoma", Font.BOLD, 30));
		theChart.getTitle().setPaint(Settings.getColor("SIMULATION_WINDOW_FONT"));
	}

	/**
	 * Ustawienie kolorw wykresu.
	 * @param theChart
	 */
	public void setColors(JFreeChart theChart)
	{
		theChart.getPlot().setBackgroundPaint(Settings.getColor("CHARTS_COLOR"));
		XYPlot plot = (XYPlot) theChart.getPlot();

		plot.getDomainAxis().setTickLabelPaint(Settings.getColor("SIMULATION_WINDOW_FONT"));
		plot.getRangeAxis().setTickLabelPaint(Settings.getColor("SIMULATION_WINDOW_FONT"));
		plot.getDomainAxis().setLabelPaint(Settings.getColor("SIMULATION_WINDOW_FONT"));
		plot.getRangeAxis().setLabelPaint(Settings.getColor("SIMULATION_WINDOW_FONT"));

		setBackground(Settings.getColor("BACKGROUND_SIMULATION"));
	}

	/**
	 * Konstruktor wykresu wartoci chwilowych.
	 * @param normalSeriesName
	 * @param mutatedSeriesName
	 * @param colorNormal
	 * @param colorMutated
	 * @param populationAmount
	 * @param icon
	 * @param title
	 */
	public Chart(String normalSeriesName, String mutatedSeriesName, Color colorNormal, Color colorMutated, int populationAmount, String icon, String title)
	{
		image1.setIcon(new ImageIcon(icon));
		image2.setIcon(new ImageIcon(icon));
		image3.setIcon(new ImageIcon(icon));

		this.normalSeries = new XYSeries(normalSeriesName, true, true);
		this.mutatedSeries = new XYSeries(mutatedSeriesName, true, true);
		dataset = new XYSeriesCollection(this.normalSeries);

		dataset.addSeries(mutatedSeries);
		theChart = createChart(dataset);
		chartPanel = new ChartPanel(theChart);
		addMouseListener();
		setLayout(null);

		add(image1);
		image1.setSize(48, 48);
		image1.setLocation(595, 45);

		add(image2);
		image2.setSize(48, 48);
		image2.setLocation(595, 103);

		add(image3);
		image3.setSize(48, 48);
		image3.setLocation(595, 161);

		add(chartPanel);
		chartPanel.setSize(600, 440);
		chartPanel.setLocation(0, 0);
		XYPlot plot = (XYPlot) theChart.getPlot();

		setColors(theChart);
		setTitle(title, theChart);

		plot.getRenderer().setSeriesPaint(0, colorNormal);
		plot.getRenderer().setSeriesPaint(1, colorMutated);
		plot.setNoDataMessage(Dictionary.getText("TEMPORARY_CHART_MESSAGE"));
		plot.setNoDataMessageFont(new Font("Tahoma", Font.BOLD, 40));
		plot.setNoDataMessagePaint(Settings.getColor("SIMULATION_WINDOW_FONT"));

		theChart.getXYPlot().getRangeAxis().setRange(new Range(-populationAmount/20,1.05*populationAmount));
	}

	/**
	 * Stwrz wykres.
	 * @param dataset
	 */
	private JFreeChart createChart(final XYDataset dataset)
	{
		final JFreeChart result = ChartFactory.createXYLineChart(" ", Dictionary.getText("X_AXIS_NAME"), Dictionary.getText("Y_AXIS_NAME"),
				dataset, PlotOrientation.VERTICAL, true, true, false);

		result.setBackgroundPaint(Settings.getColor("BACKGROUND_SIMULATION"));
		return result;
	}

	public void addThiefValue(long turnNr, double value)
	{
		double turn = (double) turnNr;
		thiefSeries.add(turn, value);
	}

	public void addProducerValue(long turnNr, double value)
	{
		double turn = (double) turnNr;
		producerSeries.add(turn, value);
	}

	public void addMutatedThiefValue(long turnNr, double value)
	{
		double turn = (double) turnNr;
		mutatedThiefSeries.add(turn, value);
	}

	public void addMutatedProducerValue(long turnNr, double value)
	{
		double turn = (double) turnNr;
		mutatedProducerSeries.add(turn, value);
	}

	public void addNormalValue(long turnNr, double value)
	{
		double turn = (double) turnNr;
		normalSeries.add(turn, value);
	}

	public void addMutatedValue(long turnNr, double value)
	{
		double turn = (double) turnNr;
		mutatedSeries.add(turn, value);

	}

	/**
	 * Wyczy wykres.
	 */
	public void clear()
	{
		if (thiefSeries != null)
			thiefSeries.clear();
		if (producerSeries != null)
			producerSeries.clear();
		if (mutatedThiefSeries != null)
			mutatedThiefSeries.clear();
		if (mutatedProducerSeries != null)
			mutatedProducerSeries.clear();
		if (normalSeries != null)
			normalSeries.clear();
		if (mutatedSeries != null)
			mutatedSeries.clear();
	}

	public PopulationProportionPane getProportionPane() {
		return proportionPane;
	}

	public void setProportionPane(PopulationProportionPane proportionPane) {
		this.proportionPane = proportionPane;
	}
}
