package view;

import java.awt.Color;
import java.awt.Desktop;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.Scrollbar;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Properties;
import java.util.regex.Pattern;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import logic.Params;
import logic.Simulation;
import settings.Dictionary;
import settings.Settings;

import java.awt.SystemColor;
import javax.swing.border.MatteBorder;
import javax.swing.text.NumberFormatter;
import javax.swing.JSpinner;

/**
 * Maa ewolucja - Gra ewolucyjno spoeczna
 *
 * 	Okno gowne programu.
 *
 * @author Grzegorz Przytua, Adam Makowski
 * @version 28092016 14:55
 */
@SuppressWarnings("serial")
public class MainWindow extends JFrame {

	private static final Pattern DOUBLE_PATTERN = Pattern
			.compile("[\\x00-\\x20]*[+-]?(NaN|Infinity|((((\\p{Digit}+)(\\.)?((\\p{Digit}+)?)"
					+ "([eE][+-]?(\\p{Digit}+))?)|(\\.((\\p{Digit}+))([eE][+-]?(\\p{Digit}+))?)|"
					+ "(((0[xX](\\p{XDigit}+)(\\.)?)|(0[xX](\\p{XDigit}+)?(\\.)(\\p{XDigit}+)))"
					+ "[pP][+-]?(\\p{Digit}+)))[fFdD]?))[\\x00-\\x20]*");

	/**
	 * Licznik stworzonych raportw.
	 */
	public static int dirCounter;
	/**
	 * Elementy GUI
	 */
	private JPanel contentPane;
	private JPanel producerPane;
	private JPanel populationPane;
	private JPanel thiefPane;
	private JPanel mutationPane;
	private JPanel otherSettingsPane;

	private JButton btnSetDefaultValue;
	private JButton btnStartSimulation;
	private JButton instructionButton;
	private JButton moreInfoButton;
	private JPanel matrix;

	private JTextField tabThief1;
	private JTextField tabProducer1;
	private JTextField tabThief2;
	private JTextField tabProducer2;
	private JTextField tabThiefThief1;
	private JTextField tabThiefProducer2;
	private JTextField tabThiefProducer1;
	private JTextField tabProducerThief1;
	private JTextField tabProducerProducer1;
	private JTextField tabProducerProducer2;
	private JTextField tabThiefThief2;
	private JTextField tabProducerThief2;
	private JTextField tabThiefThiefValue1;
	private JTextField tabThiefThiefValue2;
	private JTextField tabProducerThiefValue1;
	private JTextField tabThiefProducerValue1;
	private JTextField tabThiefProducerValue2;
	private JTextField tabProducerThiefValue2;
	private JTextField tabProducerProducerValue1;
	private JTextField tabProducerProducerValue2;
	private JTextField tabMutateThiefThiefValue;
	private JTextField tabMutateThiefProducerValue;
	private JTextField tabMutateProducerThiefValue;
	private JTextField tabMutateProducerProducerValue;
	private JTextField thiefsAmount;
	private JTextField producersAmount;
	private JTextField populationAmount;
	private JTextField mutateThiefValue;
	private JTextField mutateProducerValue;
	private JTextField txtMutatedThief;
	private JTextField txtMutatedProducer;
	private JTextField strengthSelectionText;

	private JRadioButton rdbtnMutateThief;
	private JRadioButton rdbtnMutateProducer;
	private JRadioButton rdbtnMutateThiefWithTextFields;
	private JRadioButton rdbtnMutateProducerWithTextFields;

	private Scrollbar thiefsScrollBar;
	private Scrollbar producersScrollBar;
	private Scrollbar selectionDayScrollbar;

	private JLabel matrixTitle;
	private JLabel lblMainTitle;
	private JLabel lblAuthors;
	private JLabel percent;
	private JPanel lengthOfCyclePane;
	private JLabel lblFequency;
	private JLabel lengthOfCycleFromLbl;
	private JLabel lengthOfCycleToLbl;
	private JLabel lblMutation;
	private JLabel labelPercent1;
	private JLabel labelPercent2;
	private JLabel lblOthersettingslabel;
	private JLabel populationTitle;
	private JLabel lblThiefAmount;
	private JLabel lblProducersAmount;
	private JLabel lblPopulationAmount;
	private JLabel strengthSelection;
	private JLabel lengthOfCycle;

	private JSpinner lengthOfCycleFrom;
	private JSpinner lengthOfCycleTo;

	/**
	 * Funkcja uruchamiajca symulacje i wczytujca parametry.
	 */
	public static void main(String[] args) {
		Settings.loadParams();
		Settings.loadPalette();
		Settings.loadColors();
		Dictionary.loadDictionary();

		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					MainWindow frame = new MainWindow();
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Stwrz okienko.
	 */
	public MainWindow() {
		super(Dictionary.getText("SMALL_EVOLUTION"));
		dirCounter = 0;
		setBackground(new Color(255, 255, 0));

		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		setBounds(100, 100, 1175, 892);
		contentPane = new JPanel();
		contentPane.setBackground(Settings.getColor("BACKGROUND"));
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);
		contentPane.setLayout(null);
		setResizable(false);

		ImageIcon img = new ImageIcon(ViewConsts.ICON_PATH);
		setIconImage(img.getImage());

		lblMainTitle = new JLabel(Dictionary.getText("TITLE"));
		lblMainTitle.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		lblMainTitle.setBackground(Color.WHITE);
		lblMainTitle.setFont(new Font("Tahoma", Font.BOLD, 36));
		lblMainTitle.setHorizontalAlignment(SwingConstants.CENTER);
		lblMainTitle.setBounds(10, 11, 1135, 43);
		contentPane.add(lblMainTitle);

		matrix = new JPanel();
		matrix.setBackground(Settings.getColor("MATRIX_COLOR"));
		matrix.setBounds(20, 139, 1135, 296);

		contentPane.add(matrix);
		matrix.setLayout(null);

		matrixTitle = new JLabel(Dictionary.getText("MATRIX"));
		matrixTitle.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		matrixTitle.setFont(new Font("Tahoma", Font.BOLD, 30));
		matrixTitle.setHorizontalAlignment(SwingConstants.CENTER);
		matrixTitle.setBounds(20, 11, 1090, 36);
		matrix.add(matrixTitle);

		tabThief1 = new JTextField();
		tabThief1.setBorder(new MatteBorder(0, 0, 0, 72, new ImageIcon(ViewConsts.THIEF_ICON72_V2)));
		tabThief1.setToolTipText("");
		tabThief1.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabThief1.setFont(new Font("Tahoma", Font.PLAIN, 24));
		tabThief1.setHorizontalAlignment(SwingConstants.CENTER);
		tabThief1.setText(Dictionary.getText("THIEF"));
		tabThief1.setBounds(329, 58, 288, 72);
		tabThief1.setEnabled(false);
		matrix.add(tabThief1);
		tabThief1.setColumns(10);
		tabThief1.setBackground(Settings.getColor("THIEF_COLOR"));

		tabProducer1 = new JTextField();
		tabProducer1.setBorder(new MatteBorder(0, 0, 0, 72, new ImageIcon(ViewConsts.PRODUCER_ICON72_V2)));
		tabProducer1.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabProducer1.setFont(new Font("Tahoma", Font.PLAIN, 24));
		tabProducer1.setHorizontalAlignment(SwingConstants.CENTER);
		tabProducer1.setText(Dictionary.getText("PRODUCER"));
		tabProducer1.setBounds(722, 58, 288, 72);
		tabProducer1.setEnabled(false);
		matrix.add(tabProducer1);
		tabProducer1.setColumns(10);
		tabProducer1.setBackground(Settings.getColor("PRODUCER_COLOR"));

		tabThief2 = new JTextField();
		tabThief2.setBorder(new MatteBorder(0, 0, 0, 72, new ImageIcon(ViewConsts.THIEF_ICON72_V2)));
		tabThief2.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabThief2.setFont(new Font("Tahoma", Font.PLAIN, 24));
		tabThief2.setHorizontalAlignment(SwingConstants.CENTER);
		tabThief2.setText(Dictionary.getText("THIEF"));
		tabThief2.setColumns(10);
		tabThief2.setBounds(31, 134, 288, 72);
		tabThief2.setEnabled(false);
		matrix.add(tabThief2);
		tabThief2.setBackground(Settings.getColor("THIEF_COLOR"));

		tabProducer2 = new JTextField();
		tabProducer2.setBorder(new MatteBorder(0, 0, 0, 72, new ImageIcon(ViewConsts.PRODUCER_ICON72_V2)));
		tabProducer2.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabProducer2.setFont(new Font("Tahoma", Font.PLAIN, 24));
		tabProducer2.setHorizontalAlignment(SwingConstants.CENTER);
		tabProducer2.setText(Dictionary.getText("PRODUCER"));
		tabProducer2.setColumns(10);
		tabProducer2.setBounds(31, 213, 288, 72);
		tabProducer2.setEnabled(false);
		matrix.add(tabProducer2);
		tabProducer2.setBackground(Settings.getColor("PRODUCER_COLOR"));

		tabThiefThiefValue1 = new JTextField();
		tabThiefThiefValue1.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		tabThiefThiefValue1.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabThiefThiefValue1.setFont(new Font("Tahoma", Font.PLAIN, 15));
		tabThiefThiefValue1.setHorizontalAlignment(SwingConstants.CENTER);
		tabThiefThiefValue1.setText(String.valueOf(ViewConsts.DEF_VALUE_THIEFTHIEF));
		tabThiefThiefValue1.setBounds(551, 134, 66, 34);
		matrix.add(tabThiefThiefValue1);
		tabThiefThiefValue1.setColumns(10);

		tabThiefThiefValue2 = new JTextField();
		tabThiefThiefValue2.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		tabThiefThiefValue2.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabThiefThiefValue2.setFont(new Font("Tahoma", Font.PLAIN, 15));
		tabThiefThiefValue2.setHorizontalAlignment(SwingConstants.CENTER);
		tabThiefThiefValue2.setText(String.valueOf(ViewConsts.DEF_VALUE_THIEFTHIEF));
		tabThiefThiefValue2.setBounds(551, 172, 66, 34);
		matrix.add(tabThiefThiefValue2);
		tabThiefThiefValue2.setColumns(10);

		tabProducerThiefValue1 = new JTextField();
		tabProducerThiefValue1.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		tabProducerThiefValue1.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabProducerThiefValue1.setFont(new Font("Tahoma", Font.PLAIN, 15));
		tabProducerThiefValue1.setHorizontalAlignment(SwingConstants.CENTER);
		tabProducerThiefValue1.setText(String.valueOf(ViewConsts.DEF_VALUE_PRODUCERTHIEF));
		tabProducerThiefValue1.setColumns(10);
		tabProducerThiefValue1.setBounds(551, 213, 66, 34);
		matrix.add(tabProducerThiefValue1);

		tabThiefProducerValue1 = new JTextField();
		tabThiefProducerValue1.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		tabThiefProducerValue1.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabThiefProducerValue1.setFont(new Font("Tahoma", Font.PLAIN, 15));
		tabThiefProducerValue1.setText(String.valueOf(ViewConsts.DEF_VALUE_THIEFPRODUCER));
		tabThiefProducerValue1.setHorizontalAlignment(SwingConstants.CENTER);
		tabThiefProducerValue1.setColumns(10);
		tabThiefProducerValue1.setBounds(551, 251, 66, 34);
		matrix.add(tabThiefProducerValue1);

		tabThiefProducerValue2 = new JTextField();
		tabThiefProducerValue2.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		tabThiefProducerValue2.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabThiefProducerValue2.setFont(new Font("Tahoma", Font.PLAIN, 15));
		tabThiefProducerValue2.setHorizontalAlignment(SwingConstants.CENTER);
		tabThiefProducerValue2.setText(String.valueOf(ViewConsts.DEF_VALUE_THIEFPRODUCER));
		tabThiefProducerValue2.setColumns(10);
		tabThiefProducerValue2.setBounds(944, 134, 66, 34);
		matrix.add(tabThiefProducerValue2);

		tabProducerThiefValue2 = new JTextField();
		tabProducerThiefValue2.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		tabProducerThiefValue2.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabProducerThiefValue2.setFont(new Font("Tahoma", Font.PLAIN, 15));
		tabProducerThiefValue2.setHorizontalAlignment(SwingConstants.CENTER);
		tabProducerThiefValue2.setText(String.valueOf(ViewConsts.DEF_VALUE_PRODUCERTHIEF));
		tabProducerThiefValue2.setColumns(10);
		tabProducerThiefValue2.setBounds(944, 172, 66, 34);
		matrix.add(tabProducerThiefValue2);

		tabProducerProducerValue1 = new JTextField();
		tabProducerProducerValue1.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		tabProducerProducerValue1.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabProducerProducerValue1.setFont(new Font("Tahoma", Font.PLAIN, 15));
		tabProducerProducerValue1.setText(String.valueOf(ViewConsts.DEF_VALUE_PRODUCERPRODUCER));
		tabProducerProducerValue1.setHorizontalAlignment(SwingConstants.CENTER);
		tabProducerProducerValue1.setColumns(10);
		tabProducerProducerValue1.setBounds(944, 213, 66, 34);
		matrix.add(tabProducerProducerValue1);

		tabProducerProducerValue2 = new JTextField();
		tabProducerProducerValue2.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		tabProducerProducerValue2.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabProducerProducerValue2.setFont(new Font("Tahoma", Font.PLAIN, 15));
		tabProducerProducerValue2.setHorizontalAlignment(SwingConstants.CENTER);
		tabProducerProducerValue2.setText(String.valueOf(ViewConsts.DEF_VALUE_PRODUCERPRODUCER));
		tabProducerProducerValue2.setColumns(10);
		tabProducerProducerValue2.setBounds(944, 251, 66, 34);
		matrix.add(tabProducerProducerValue2);

		tabThiefThief1 = new JTextField();
		tabThiefThief1.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabThiefThief1.setFont(new Font("Tahoma", Font.PLAIN, 16));
		tabThiefThief1.setHorizontalAlignment(SwingConstants.CENTER);
		tabThiefThief1.setText(parseMyText(Dictionary.getText("THIEF3"), tabThiefThiefValue1.getText()));
		tabThiefThief1.setBounds(329, 134, 218, 34);
		tabThiefThief1.setEnabled(false);
		matrix.add(tabThiefThief1);
		tabThiefThief1.setColumns(10);

		tabThiefProducer2 = new JTextField();
		tabThiefProducer2.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabThiefProducer2.setFont(new Font("Tahoma", Font.PLAIN, 16));
		tabThiefProducer2.setHorizontalAlignment(SwingConstants.CENTER);
		tabThiefProducer2.setText(parseMyText(Dictionary.getText("THIEF3"), tabThiefProducerValue1.getText()));
		tabThiefProducer2.setColumns(10);
		tabThiefProducer2.setBounds(329, 251, 218, 34);
		tabThiefProducer2.setEnabled(false);
		matrix.add(tabThiefProducer2);

		tabThiefProducer1 = new JTextField();
		tabThiefProducer1.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabThiefProducer1.setFont(new Font("Tahoma", Font.PLAIN, 16));
		tabThiefProducer1.setHorizontalAlignment(SwingConstants.CENTER);
		tabThiefProducer1.setText(parseMyText(Dictionary.getText("THIEF3"), tabThiefProducerValue1.getText()));
		tabThiefProducer1.setColumns(10);
		tabThiefProducer1.setBounds(722, 134, 218, 34);
		tabThiefProducer1.setEnabled(false);
		matrix.add(tabThiefProducer1);

		tabProducerThief1 = new JTextField();
		tabProducerThief1.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabProducerThief1.setFont(new Font("Tahoma", Font.PLAIN, 16));
		tabProducerThief1.setHorizontalAlignment(SwingConstants.CENTER);
		tabProducerThief1.setText(parseMyText(Dictionary.getText("PRODUCER3"), tabProducerThiefValue1.getText()));
		tabProducerThief1.setColumns(10);
		tabProducerThief1.setBounds(722, 173, 218, 34);
		tabProducerThief1.setEnabled(false);
		matrix.add(tabProducerThief1);

		tabProducerProducer1 = new JTextField();
		tabProducerProducer1.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabProducerProducer1.setFont(new Font("Tahoma", Font.PLAIN, 16));
		tabProducerProducer1.setHorizontalAlignment(SwingConstants.CENTER);
		tabProducerProducer1.setText(parseMyText(Dictionary.getText("PRODUCER3"), tabProducerProducerValue1.getText()));
		tabProducerProducer1.setColumns(10);
		tabProducerProducer1.setBounds(722, 213, 218, 34);
		tabProducerProducer1.setEnabled(false);
		matrix.add(tabProducerProducer1);

		tabProducerProducer2 = new JTextField();
		tabProducerProducer2.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabProducerProducer2.setFont(new Font("Tahoma", Font.PLAIN, 16));
		tabProducerProducer2.setHorizontalAlignment(SwingConstants.CENTER);
		tabProducerProducer2.setText(parseMyText(Dictionary.getText("PRODUCER3"), tabProducerProducerValue1.getText()));
		tabProducerProducer2.setColumns(10);
		tabProducerProducer2.setBounds(722, 251, 218, 34);
		tabProducerProducer2.setEnabled(false);
		matrix.add(tabProducerProducer2);

		tabThiefThief2 = new JTextField();
		tabThiefThief2.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabThiefThief2.setFont(new Font("Tahoma", Font.PLAIN, 16));
		tabThiefThief2.setHorizontalAlignment(SwingConstants.CENTER);
		tabThiefThief2.setText(parseMyText(Dictionary.getText("THIEF3"), tabThiefThiefValue1.getText()));
		tabThiefThief2.setColumns(10);
		tabThiefThief2.setBounds(329, 172, 218, 34);
		tabThiefThief2.setEnabled(false);
		matrix.add(tabThiefThief2);

		tabProducerThief2 = new JTextField();
		tabProducerThief2.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabProducerThief2.setFont(new Font("Tahoma", Font.PLAIN, 16));
		tabProducerThief2.setHorizontalAlignment(SwingConstants.CENTER);
		tabProducerThief2.setText(parseMyText(Dictionary.getText("PRODUCER3"), tabProducerThiefValue1.getText()));
		tabProducerThief2.setColumns(10);
		tabProducerThief2.setBounds(329, 213, 218, 34);
		tabProducerThief2.setEnabled(false);
		matrix.add(tabProducerThief2);

		tabMutateThiefThiefValue = new JTextField();
		tabMutateThiefThiefValue.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabMutateThiefThiefValue.setEnabled(false);
		tabMutateThiefThiefValue.setHorizontalAlignment(SwingConstants.CENTER);
		tabMutateThiefThiefValue.setFont(new Font("Tahoma", Font.PLAIN, 20));
		tabMutateThiefThiefValue.setColumns(10);
		tabMutateThiefThiefValue.setBounds(627, 134, 85, 72);
		tabMutateThiefThiefValue.setVisible(false);
		matrix.add(tabMutateThiefThiefValue);

		tabMutateThiefProducerValue = new JTextField();
		tabMutateThiefProducerValue.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabMutateThiefProducerValue.setEnabled(false);
		tabMutateThiefProducerValue.setHorizontalAlignment(SwingConstants.CENTER);
		tabMutateThiefProducerValue.setFont(new Font("Tahoma", Font.PLAIN, 20));
		tabMutateThiefProducerValue.setColumns(10);
		tabMutateThiefProducerValue.setBounds(627, 213, 85, 72);
		tabMutateThiefProducerValue.setVisible(false);
		matrix.add(tabMutateThiefProducerValue);

		tabMutateProducerThiefValue = new JTextField();
		tabMutateProducerThiefValue.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabMutateProducerThiefValue.setEnabled(false);
		tabMutateProducerThiefValue.setHorizontalAlignment(SwingConstants.CENTER);
		tabMutateProducerThiefValue.setFont(new Font("Tahoma", Font.PLAIN, 20));
		tabMutateProducerThiefValue.setColumns(10);
		tabMutateProducerThiefValue.setBounds(1020, 134, 85, 72);
		tabMutateProducerThiefValue.setVisible(false);
		matrix.add(tabMutateProducerThiefValue);

		tabMutateProducerProducerValue = new JTextField();
		tabMutateProducerProducerValue.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		tabMutateProducerProducerValue.setEnabled(false);
		tabMutateProducerProducerValue.setHorizontalAlignment(SwingConstants.CENTER);
		tabMutateProducerProducerValue.setFont(new Font("Tahoma", Font.PLAIN, 20));
		tabMutateProducerProducerValue.setColumns(10);
		tabMutateProducerProducerValue.setBounds(1020, 213, 85, 72);
		tabMutateProducerProducerValue.setVisible(false);
		matrix.add(tabMutateProducerProducerValue);

		calculateMutateThiefBonus(String.valueOf(ViewConsts.DEF_VALUE_PERCENT_MUTATION));
		calculateMutateProducerBonus(String.valueOf(ViewConsts.DEF_VALUE_PERCENT_MUTATION));

		txtMutatedThief = new JTextField();
		txtMutatedThief.setBorder(null);
		txtMutatedThief.setToolTipText(Dictionary.getText("MUTATED_THIEF"));
		txtMutatedThief.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		txtMutatedThief.setText(Dictionary.getText("MUTATED_THIEF_SHORT"));
		txtMutatedThief.setHorizontalAlignment(SwingConstants.CENTER);
		txtMutatedThief.setFont(new Font("Tahoma", Font.PLAIN, 24));
		txtMutatedThief.setEnabled(false);
		txtMutatedThief.setColumns(10);
		txtMutatedThief.setBounds(628, 58, 85, 72);
		txtMutatedThief.setVisible(false);
		matrix.add(txtMutatedThief);
		txtMutatedThief.setBackground(Settings.getColor("MUTATED_THIEF_COLOR"));

		txtMutatedProducer = new JTextField();
		txtMutatedProducer.setBorder(null);
		txtMutatedProducer.setToolTipText(Dictionary.getText("MUTATED_PRODUCER"));
		txtMutatedProducer.setDisabledTextColor(Settings.getColor("MAIN_WINDOW_FONT"));
		txtMutatedProducer.setText(Dictionary.getText("MUTATED_PRODUCER_SHORT"));
		txtMutatedProducer.setHorizontalAlignment(SwingConstants.CENTER);
		txtMutatedProducer.setFont(new Font("Tahoma", Font.PLAIN, 24));
		txtMutatedProducer.setEnabled(false);
		txtMutatedProducer.setColumns(10);
		txtMutatedProducer.setBounds(1020, 58, 85, 72);
		txtMutatedProducer.setVisible(false);
		matrix.add(txtMutatedProducer);
		txtMutatedProducer.setBackground(Settings.getColor("MUTATED_PRODUCER_COLOR"));

		btnSetDefaultValue = new JButton(Dictionary.getText("DEFAULT_VALUES"));
		btnSetDefaultValue.setFont(new Font("Tahoma", Font.BOLD, 12));
		btnSetDefaultValue.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		btnSetDefaultValue.setBackground(Settings.getColor("DEFAULT_VALUES_COLOR"));
		btnSetDefaultValue.setBounds(31, 58, 288, 72);
		btnSetDefaultValue
				.setToolTipText("<html><p width=\"100\">" + Dictionary.getText("DEFAULT_VALUES_INFO") + "</p></html>");
		matrix.add(btnSetDefaultValue);

		populationPane = new JPanel();
		populationPane.setBounds(20, 444, 1135, 162);
		populationPane.setBackground(Settings.getColor("POPULATION_COLOR"));
		contentPane.add(populationPane);

		thiefPane = new JPanel();
		thiefPane.setBounds(10, 52, 841, 43);
		populationPane.add(thiefPane);
		thiefPane.setBackground(new Color(220, 220, 220));
		thiefPane.setLayout(null);
		thiefPane.setBackground(Settings.getColor("THIEF_PANE_COLOR"));

		producerPane = new JPanel();
		producerPane.setBackground(new Color(220, 220, 220));
		producerPane.setBounds(10, 106, 841, 43);
		populationPane.add(producerPane);
		producerPane.setLayout(null);
		producerPane.setBackground(Settings.getColor("PRODUCER_PANE_COLOR"));

		mutationPane = new JPanel();
		mutationPane.setBounds(20, 611, 483, 140);
		contentPane.add(mutationPane);
		mutationPane.setLayout(null);
		mutationPane.setBackground(Settings.getColor("MUTATION_COLOR"));

		otherSettingsPane = new JPanel();
		otherSettingsPane.setBounds(513, 611, 367, 140);
		contentPane.add(otherSettingsPane);
		otherSettingsPane.setLayout(null);
		otherSettingsPane.setBackground(Settings.getColor("SELECTION_STRENGTH_COLOR"));

		populationTitle = new JLabel(Dictionary.getText("POPULATION"));
		populationTitle.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		populationTitle.setHorizontalAlignment(SwingConstants.CENTER);
		populationTitle.setFont(new Font("Tahoma", Font.BOLD, 30));
		populationTitle.setSize(1115, 36);
		populationTitle.setLocation(10, 5);

		populationPane.setLayout(null);
		populationPane.add(populationTitle);

		thiefsAmount = new JTextField();
		thiefsAmount.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		thiefsAmount.setBackground(Settings.getColor("THIEFS_AMMOUNT_FIELD"));
		thiefsAmount.setFont(new Font("Tahoma", Font.PLAIN, 23));
		thiefsAmount.setHorizontalAlignment(SwingConstants.CENTER);
		thiefsAmount.setText(String.valueOf(ViewConsts.DEF_VALUE_SPIECES_AMOUNT));
		thiefsAmount.setBounds(/* 453 */689, 7, 142, 29);
		thiefPane.add(thiefsAmount);
		thiefsAmount.setColumns(10);

		producersAmount = new JTextField();
		producersAmount.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		producersAmount.setBackground(Settings.getColor("PRODUCERS_AMMOUNT_FIELD"));
		producersAmount.setFont(new Font("Tahoma", Font.PLAIN, 23));
		producersAmount.setHorizontalAlignment(SwingConstants.CENTER);
		producersAmount.setText(String.valueOf(ViewConsts.DEF_VALUE_SPIECES_AMOUNT));
		producersAmount.setColumns(10);
		producersAmount.setBounds(689, 7, 142, 29);
		producerPane.add(producersAmount);

		thiefsScrollBar = new Scrollbar();
		thiefsScrollBar.setBackground(Settings.getColor("SCROLLS"));
		thiefsScrollBar.setVisibleAmount(0);
		thiefsScrollBar.setMinimum(1);
		thiefsScrollBar.setMaximum(1000);
		thiefsScrollBar.setOrientation(Scrollbar.HORIZONTAL);
		thiefsScrollBar.setBounds(167, 7, /* 274 */ 516, 29);
		thiefsScrollBar.setValue(Integer.valueOf(thiefsAmount.getText()));
		thiefPane.add(thiefsScrollBar);

		producersScrollBar = new Scrollbar();
		producersScrollBar.setBackground(Settings.getColor("SCROLLS"));
		producersScrollBar.setVisibleAmount(0);
		producersScrollBar.setMinimum(1);
		producersScrollBar.setMaximum(1000);
		producersScrollBar.setOrientation(Scrollbar.HORIZONTAL);
		producersScrollBar.setBounds(167, 7, 516, 29);
		producersScrollBar.setValue(Integer.valueOf(producersAmount.getText()));
		producerPane.add(producersScrollBar);

		lblThiefAmount = new JLabel(Dictionary.getText("THIEFS_AMOUNT"));
		lblThiefAmount.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		lblThiefAmount.setBackground(new Color(240, 255, 240));
		lblThiefAmount.setFont(new Font("Tahoma", Font.PLAIN, 18));
		lblThiefAmount.setHorizontalAlignment(SwingConstants.CENTER);
		lblThiefAmount.setBounds(10, 7, 151, 29);
		thiefPane.add(lblThiefAmount);

		lblProducersAmount = new JLabel(Dictionary.getText("PRODUCERS_AMOUNT"));
		lblProducersAmount.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		lblProducersAmount.setHorizontalAlignment(SwingConstants.CENTER);
		lblProducersAmount.setFont(new Font("Tahoma", Font.PLAIN, 18));
		lblProducersAmount.setBounds(10, 7, 151, 29);
		producerPane.add(lblProducersAmount);

		populationAmount = new JTextField();
		populationAmount.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		populationAmount.setBackground(Settings.getColor("POPULATION_AMMOUNT_FIELD"));
		populationAmount.setFont(new Font("Tahoma", Font.PLAIN, 25));
		populationAmount.setText(String.valueOf(producersScrollBar.getValue() + thiefsScrollBar.getValue()));
		populationAmount.setHorizontalAlignment(SwingConstants.CENTER);
		populationAmount.setBounds(874, 115, 251, 36);
		populationPane.add(populationAmount);
		populationAmount.setColumns(10);

		lblPopulationAmount = new JLabel(Dictionary.getText("POPULATION_AMOUNT"));
		lblPopulationAmount.setFont(new Font("Tahoma", Font.PLAIN, 24));
		lblPopulationAmount.setHorizontalAlignment(SwingConstants.CENTER);
		lblPopulationAmount.setBounds(874, 52, 251, 52);
		lblPopulationAmount.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		populationPane.add(lblPopulationAmount);

		lblMutation = new JLabel(Dictionary.getText("MUTATION_LBL"));
		lblMutation.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		lblMutation.setFont(new Font("Tahoma", Font.BOLD, 30));
		lblMutation.setHorizontalAlignment(SwingConstants.CENTER);
		lblMutation.setBounds(10, 5, 437, 36);
		lblMutation.setToolTipText("<html><p width=\"100\">" + Dictionary.getText("MUTANT_INFO") + "</p></html>");
		mutationPane.add(lblMutation);

		mutateThiefValue = new JTextField();
		mutateThiefValue.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		mutateThiefValue.setEnabled(false);
		mutateThiefValue.setText(String.valueOf(ViewConsts.DEF_VALUE_PERCENT_MUTATION));
		mutateThiefValue.setHorizontalAlignment(SwingConstants.CENTER);
		mutateThiefValue.setFont(new Font("Tahoma", Font.PLAIN, 15));
		mutateThiefValue.setColumns(10);
		mutateThiefValue.setBounds(176, 48, 55, 35);
		mutationPane.add(mutateThiefValue);

		mutateProducerValue = new JTextField();
		mutateProducerValue.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		mutateProducerValue.setEnabled(false);
		mutateProducerValue.setText(String.valueOf(ViewConsts.DEF_VALUE_PERCENT_MUTATION));
		mutateProducerValue.setHorizontalAlignment(SwingConstants.CENTER);
		mutateProducerValue.setFont(new Font("Tahoma", Font.PLAIN, 15));
		mutateProducerValue.setColumns(10);
		mutateProducerValue.setBounds(176, 96, 55, 35);
		mutationPane.add(mutateProducerValue);

		rdbtnMutateThief = new JRadioButton(Dictionary.getText("RDB_THIEF_PERCENT"));
		rdbtnMutateThief.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		rdbtnMutateThief.setBounds(10, 48, 160, 35);
		mutationPane.add(rdbtnMutateThief);
		rdbtnMutateThief
				.setToolTipText("<html><p width=\"100\">" + Dictionary.getText("RDB_THIEF_INFO_TXTPN") + "</p></html>");
		rdbtnMutateThief.setBackground(Settings.getColor("MUTATION_COLOR"));

		rdbtnMutateProducer = new JRadioButton(Dictionary.getText("RDB_PRODUCER_PERCENT"));
		rdbtnMutateProducer.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		rdbtnMutateProducer.setBounds(10, 96, 160, 35);
		mutationPane.add(rdbtnMutateProducer);
		rdbtnMutateProducer.setToolTipText(
				"<html><p width=\"100\">" + Dictionary.getText("RDB_PRODUCER_INFO_TXTPN") + "</p></html>");
		rdbtnMutateProducer.setBackground(Settings.getColor("MUTATION_COLOR"));

		rdbtnMutateThiefWithTextFields = new JRadioButton(Dictionary.getText("RDB_THIEF_MANUALLY"));
		rdbtnMutateThiefWithTextFields.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		rdbtnMutateThiefWithTextFields.setBounds(268, 48, 209, 35);
		mutationPane.add(rdbtnMutateThiefWithTextFields);
		rdbtnMutateThiefWithTextFields.setBackground(Settings.getColor("MUTATION_COLOR"));
		rdbtnMutateThiefWithTextFields.setToolTipText("<html><p width=\"100\">" + Dictionary.getText("RDB_THIEF_MANUALY_INFO_TXTPN") + "</p></html>");

		rdbtnMutateProducerWithTextFields = new JRadioButton(Dictionary.getText("RDB_PRODUCER_MANUALLY"));
		rdbtnMutateProducerWithTextFields.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		rdbtnMutateProducerWithTextFields.setBounds(268, 96, 209, 35);
		mutationPane.add(rdbtnMutateProducerWithTextFields);
		rdbtnMutateProducerWithTextFields.setBackground(Settings.getColor("MUTATION_COLOR"));
		rdbtnMutateProducerWithTextFields.setToolTipText("<html><p width=\"100\">" + Dictionary.getText("RDB_PRODUCER_MANUALY_INFO_TXTPN") + "</p></html>");

		labelPercent1 = new JLabel("%");
		labelPercent1.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		labelPercent1.setFont(new Font("Tahoma", Font.PLAIN, 20));
		labelPercent1.setHorizontalAlignment(SwingConstants.CENTER);
		labelPercent1.setBounds(235, 48, 27, 35);
		mutationPane.add(labelPercent1);

		labelPercent2 = new JLabel("%");
		labelPercent2.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		labelPercent2.setHorizontalAlignment(SwingConstants.CENTER);
		labelPercent2.setFont(new Font("Tahoma", Font.PLAIN, 20));
		labelPercent2.setBounds(235, 96, 27, 35);
		mutationPane.add(labelPercent2);

		lblOthersettingslabel = new JLabel(Dictionary.getText("SELECTION_STRENGTH"));
		lblOthersettingslabel.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		lblOthersettingslabel.setHorizontalAlignment(SwingConstants.CENTER);
		lblOthersettingslabel.setFont(new Font("Tahoma", Font.BOLD, 30));
		lblOthersettingslabel.setBounds(10, 5, 349, 36);
		otherSettingsPane.add(lblOthersettingslabel);

		btnStartSimulation = new JButton(Dictionary.getText("SIMULATION_START"));
		btnStartSimulation.setBounds(890, 611, 265, 230);
		contentPane.add(btnStartSimulation);
		btnStartSimulation.setBackground(Settings.getColor("START_COLOR"));
		btnStartSimulation.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		btnStartSimulation.setFont(new Font("Tahoma", Font.BOLD, 22));
		btnStartSimulation
				.setToolTipText("<html><p width=\"100\">" + Dictionary.getText("SIMULATION_INFO") + "</p></html>");

		instructionButton = new JButton(Dictionary.getText("INSTRUCTION"));
		instructionButton.setBounds(20, 74, 314, 54);
		contentPane.add(instructionButton);
		instructionButton.setFont(new Font("Tahoma", Font.BOLD, 14));
		instructionButton.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		instructionButton.setBackground(Settings.getColor("INSTRUCTION_COLOR"));

		lblAuthors = new JLabel("<html><center>"+Dictionary.getText("AUTHOR")+" Jan Kuba " + Dictionary.getText("IMPLEMENTATION") + " Grzegorz Przytua, Adam Makowski</center></html>");
		lblAuthors.setForeground(new Color(192, 192, 192));
		lblAuthors.setHorizontalAlignment(SwingConstants.RIGHT);
		lblAuthors.setFont(new Font("Tahoma", Font.BOLD, 14));
		lblAuthors.setBounds(20, 837, 1135, 26);
		contentPane.add(lblAuthors);

		moreInfoButton = new JButton("");
		moreInfoButton.setIcon(new ImageIcon(ViewConsts.HEADER_PATH + Dictionary.getLang()+".jpg"));
		moreInfoButton.setBorderPainted(false);
		moreInfoButton.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		moreInfoButton.setFont(new Font("Tahoma", Font.BOLD, 14));
		moreInfoButton.setBackground(new Color(248, 248, 255));
		moreInfoButton.setBounds(344, 74, 646, 54);
		contentPane.add(moreInfoButton);


		strengthSelectionText = new JTextField();
		strengthSelectionText.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		strengthSelectionText.setBackground(SystemColor.window);
		strengthSelectionText
				.setToolTipText("<html><p width=\"100\">" + Dictionary.getText("HOW_MUCH_DIE_INFO") + "</p></html>");
		strengthSelectionText.setText("50");
		strengthSelectionText.setHorizontalAlignment(SwingConstants.CENTER);
		strengthSelectionText.setFont(new Font("Tahoma", Font.PLAIN, 12));
		strengthSelectionText.setColumns(10);
		strengthSelectionText.setBounds(270, 77, 51, 30);
		otherSettingsPane.add(strengthSelectionText);
		strengthSelectionText
				.setToolTipText("<html><p width=\"100\">" + Dictionary.getText("HOW_MUCH_DIE_INFO") + "</p></html>");

		selectionDayScrollbar = new Scrollbar();
		selectionDayScrollbar.setVisibleAmount(0);
		selectionDayScrollbar.setValue(50);
		selectionDayScrollbar.setOrientation(Scrollbar.HORIZONTAL);
		selectionDayScrollbar.setMinimum(1);
		selectionDayScrollbar.setMaximum(100);
		selectionDayScrollbar.setBounds(10, 113, 349, 17);
		selectionDayScrollbar.setBackground(Settings.getColor("SCROLLS"));
		otherSettingsPane.add(selectionDayScrollbar);

		strengthSelection = new JLabel();
		strengthSelection.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		strengthSelection.setHorizontalAlignment(SwingConstants.RIGHT);
		strengthSelection.setToolTipText("<html><p width=\"100\">" + Dictionary.getText("HOW_MUCH_DIE_INFO") + "</p></html>");
		strengthSelection.setText(Dictionary.getText("HOW_MUCH_DIE"));
		strengthSelection.setFont(new Font("Tahoma", Font.PLAIN, 14));
		strengthSelection.setBackground(SystemColor.menu);
		strengthSelection.setBounds(10, 77, 250, 30);
		otherSettingsPane.add(strengthSelection);
		strengthSelectionText.setText(String.valueOf(selectionDayScrollbar.getValue()));

		percent = new JLabel("%");
		percent.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		percent.setHorizontalAlignment(SwingConstants.CENTER);
		percent.setFont(new Font("Tahoma", Font.PLAIN, 20));
		percent.setBounds(330, 77, 27, 30);
		otherSettingsPane.add(percent);

		lengthOfCyclePane = new JPanel();
		lengthOfCyclePane.setBounds(20, 762, 860, 79);
		lengthOfCyclePane.setBackground(Settings.getColor("LENGTH_OF_CYCLE_PANE_COLOR"));
		contentPane.add(lengthOfCyclePane);
		lengthOfCyclePane.setLayout(null);

		lblFequency = new JLabel(Dictionary.getText("LENGTH_OF_CYCLE_LBL"));
		lblFequency.setBounds(10, 10, 361, 59);
		lblFequency.setToolTipText("<html><p width=\"100\">" + Dictionary.getText("LENGTH_OF_CYCLE_INFO") + "</p></html>");
		lblFequency.setHorizontalAlignment(SwingConstants.CENTER);
		lblFequency.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		lblFequency.setFont(new Font("Tahoma", Font.BOLD, 30));
		lengthOfCyclePane.add(lblFequency);

		lengthOfCycleFromLbl = new JLabel(Dictionary.getText("LENGTH_OF_CYCLE_FROM"));
		lengthOfCycleFromLbl.setHorizontalAlignment(SwingConstants.CENTER);
		lengthOfCycleFromLbl.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		lengthOfCycleFromLbl.setFont(new Font("Tahoma", Font.PLAIN, 24));
		lengthOfCycleFromLbl.setBounds(386, 10, 98, 59);
		lengthOfCyclePane.add(lengthOfCycleFromLbl);

		lengthOfCycleToLbl = new JLabel(Dictionary.getText("LENGTH_OF_CYCLE_TO"));
		lengthOfCycleToLbl.setHorizontalAlignment(SwingConstants.CENTER);
		lengthOfCycleToLbl.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		lengthOfCycleToLbl.setFont(new Font("Tahoma", Font.PLAIN, 24));
		lengthOfCycleToLbl.setBounds(592, 10, 52, 59);
		lengthOfCyclePane.add(lengthOfCycleToLbl);

		lengthOfCycleFrom = new JSpinner();
		lengthOfCycleFrom.setBounds(494, 14, 88, 50);
		lengthOfCycleFrom.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		lengthOfCycleFrom.setFont(new Font("Tahoma", Font.PLAIN, 18));
		lengthOfCyclePane.add(lengthOfCycleFrom);

		lengthOfCycleTo = new JSpinner();
		lengthOfCycleTo.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		lengthOfCycleTo.setFont(new Font("Tahoma", Font.PLAIN, 18));
		lengthOfCycleTo.setBounds(654, 14, 88, 50);
		lengthOfCyclePane.add(lengthOfCycleTo);

		lengthOfCycle = new JLabel(Dictionary.getText("LENGTH_OF_CYCLE_ROUND"));
		lengthOfCycle.setHorizontalAlignment(SwingConstants.CENTER);
		lengthOfCycle.setForeground(Settings.getColor("MAIN_WINDOW_FONT"));
		lengthOfCycle.setFont(new Font("Tahoma", Font.PLAIN, 24));
		lengthOfCycle.setBounds(752, 10, 98, 59);
		lengthOfCyclePane.add(lengthOfCycle);

		prepareLengthOfCycleSpinners();

		File propertiesFile = new File(ViewConsts.FILE_PROPERTIES_NAME);
		if (!propertiesFile.exists())
			try {
				makeDefaultProperties();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		else
			readProperties();

		doubleAmountWrapper(tabThiefThiefValue1, -200d, 200d);
		doubleAmountWrapper(tabThiefThiefValue2, -200d, 200d);
		doubleAmountWrapper(tabThiefProducerValue1, -200d, 200d);
		doubleAmountWrapper(tabThiefProducerValue2, -200d, 200d);
		doubleAmountWrapper(tabProducerThiefValue1, -200d, 200d);
		doubleAmountWrapper(tabProducerThiefValue2, -200d, 200d);
		doubleAmountWrapper(tabProducerProducerValue1, -200d, 200d);
		doubleAmountWrapper(tabProducerProducerValue2, -200d, 200d);
		doubleAmountWrapper(tabMutateThiefThiefValue, -200d, 200d);
		doubleAmountWrapper(tabMutateThiefProducerValue, -200d, 200d);
		doubleAmountWrapper(tabMutateProducerProducerValue, -200d, 200d);
		doubleAmountWrapper(tabMutateProducerThiefValue, -200d, 200d);

		integerAmountWrapper(mutateThiefValue, 1, 100);
		integerAmountWrapper(mutateProducerValue, 1, 100);
		integerAmountWrapper(populationAmount, 2, 9999);
		integerAmountWrapper(producersAmount, 1, 5000);
		integerAmountWrapper(thiefsAmount, 1, 4999);

		JLabel gplLogo = new JLabel();
		gplLogo.setOpaque(true);
		gplLogo.setIcon(new ImageIcon(ViewConsts.LOGO));
		gplLogo.setBackground(new Color(248, 248, 255));
		gplLogo.setBounds(1000, 74, 155, 54);
		contentPane.add(gplLogo);

		/**
		 * Przycisk przwywrcenia wartoci domylnych.
		 */
		btnSetDefaultValue.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setDefaultValues();
			}
		});

		/**
		 * Otworzenie instrukcji obsugi.
		 */
		instructionButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try
				{
					if(Dictionary.getLang().equals("pol"))
						Desktop.getDesktop().browse(new URL(ViewConsts.INSTRUCTION_POL).toURI());
					else
						Desktop.getDesktop().browse(new URL(ViewConsts.INSTRUCTION_ENG).toURI());
				}
				catch (IOException e1)
				{
					e1.printStackTrace();
				}
				catch (URISyntaxException e1)
				{
					e1.printStackTrace();
				}

			}
		});

		/**
		 * Otworzenie strony projektu.
		 */
		moreInfoButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try
				{
					if(Dictionary.getLang().equals("pol"))
						Desktop.getDesktop().browse(new URL(ViewConsts.SMALL_EVOLUTION_POL).toURI());
					else
						Desktop.getDesktop().browse(new URL(ViewConsts.SMALL_EVOLUTION_ENG).toURI());
				}
				catch (IOException e1)
				{
					e1.printStackTrace();
				}
				catch (URISyntaxException e1)
				{
					e1.printStackTrace();
				}
			}
		});

		/**
		 * Uruchomienie symulacji.
		 */
		btnStartSimulation.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {

							if((int) lengthOfCycleFrom.getValue() > (int) lengthOfCycleTo.getValue())
							{
								JOptionPane.showMessageDialog(null, Dictionary.getText("START_ERROR_MESSAGE"), Dictionary.getText("START_ERROR_TITLE"), JOptionPane.ERROR_MESSAGE);
								return;
							}

							SimulationWindow frame;

							frame = new SimulationWindow(thiefsScrollBar.getValue(), producersScrollBar.getValue(),
									Double.valueOf(tabThiefThiefValue1.getText()),
									Double.valueOf(tabProducerProducerValue1.getText()),
									Double.valueOf(tabThiefProducerValue1.getText()),
									Double.valueOf(tabProducerThiefValue2.getText()),
									Integer.valueOf(strengthSelectionText.getText()),
									getLengthOfCycleFrom(),
									getLengthOfCycleTo(),
									(rdbtnMutateThief.isSelected() || rdbtnMutateThiefWithTextFields.isSelected()),
									(rdbtnMutateProducer.isSelected() || rdbtnMutateProducerWithTextFields.isSelected())
									);
							if (rdbtnMutateThief.isSelected() || rdbtnMutateThiefWithTextFields.isSelected())
								frame.mutateThief(Double.valueOf(tabMutateThiefThiefValue.getText()),
										Double.valueOf(tabMutateThiefProducerValue.getText()));
							if (rdbtnMutateProducer.isSelected() || rdbtnMutateProducerWithTextFields.isSelected())
								frame.mutateProducer(Double.valueOf(tabMutateProducerProducerValue.getText()),
										Double.valueOf(tabMutateProducerThiefValue.getText()));
							frame.setVisible(true);

						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});

		/**
		 * Ustawienie wartoci siy funkcji selekcyjnej.
		 */
		selectionDayScrollbar.addAdjustmentListener(new AdjustmentListener() {

			@Override
			public void adjustmentValueChanged(AdjustmentEvent e) {
				strengthSelectionText.setText(String.valueOf(selectionDayScrollbar.getValue()));
			}
		});

		/**
		 * Ustawienie wartoci siy funkcji selekcyjnej.
		 */
		strengthSelectionText.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				selectionDayScrollbar.setValue(Integer.valueOf(strengthSelectionText.getText()));
			}
		});

		/**
		 * Ustawienie procentowego mutowania zodzei.
		 */
		rdbtnMutateThief.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				calculateMutateThiefBonus(mutateThiefValue.getText());
				if (rdbtnMutateThief.isSelected()) {
					rdbtnMutateThiefWithTextFields.setSelected(false);
					setVisibleMutateThiefValues(true);
					mutateThiefValue.setEnabled(true);
					setEnabledMutateThiefValues(false);
				} else {
					mutateThiefValue.setEnabled(false);
					if (!rdbtnMutateThiefWithTextFields.isSelected())
						setVisibleMutateThiefValues(false);
				}
			}
		});

		/**
		 * Ustawienie procentowego mutowania producentw.
		 */
		rdbtnMutateProducer.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				calculateMutateProducerBonus(mutateProducerValue.getText());
				if (rdbtnMutateProducer.isSelected()) {
					rdbtnMutateProducerWithTextFields.setSelected(false);
					setVisibleMutateProducerValues(true);
					mutateProducerValue.setEnabled(true);
					setEnabledMutateProducerValues(false);
				} else {
					mutateProducerValue.setEnabled(false);
					if (!rdbtnMutateProducerWithTextFields.isSelected())
						setVisibleMutateProducerValues(false);
				}

			}
		});

		/**
		 * Pokazanie pl zmutowanych zodziei w macierzy wypat.
		 */
		rdbtnMutateThiefWithTextFields.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (rdbtnMutateThiefWithTextFields.isSelected()) {
					setVisibleMutateThiefValues(true);
					rdbtnMutateThief.setSelected(false);
					mutateThiefValue.setEnabled(false);
					setEnabledMutateThiefValues(true);
				} else if (!rdbtnMutateThief.isSelected())
					setVisibleMutateThiefValues(false);
			}
		});

		/**
		 * Pokazanie pl zmutowanych producentw w macierzy wypat.
		 */
		rdbtnMutateProducerWithTextFields.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (rdbtnMutateProducerWithTextFields.isSelected()) {
					setVisibleMutateProducerValues(true);
					rdbtnMutateProducer.setSelected(false);
					mutateProducerValue.setEnabled(false);
					setEnabledMutateProducerValues(true);
				} else if (!rdbtnMutateProducer.isSelected())
					setVisibleMutateProducerValues(false);
			}
		});

		/**
		 * Ustawienie zalenoci pomidzy zmutowanym a zwykym zodziejem.
		 */
		mutateThiefValue.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (mutateThiefValue.getText().length() >= 5)
					mutateThiefValue.setText("99");
				if (!mutateThiefValue.getText().isEmpty())
					calculateMutateThiefBonus(mutateThiefValue.getText());
			}
		});

		/**
		 * Ustawienie zalenoci pomidzy zmutowanym a zwykym producentem.
		 */
		mutateProducerValue.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (mutateProducerValue.getText().length() >= 5)
					mutateProducerValue.setText("99");
				if (!mutateProducerValue.getText().isEmpty())
					calculateMutateProducerBonus(mutateProducerValue.getText());
			}
		});

		/**
		 * Obsuga pola w macierzy wypat.
		 */
		tabThiefThiefValue1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (tabThiefThiefValue1.getText().equals("-") || tabThiefThiefValue1.getText().equals("."))
					tabThiefThiefValue1.setText("0.0");
				if (tabThiefThiefValue1.getText().length() >= 8)
					tabThiefThiefValue1.setText("200.0");
				if (!tabThiefThiefValue1.getText().isEmpty() && !(tabThiefThiefValue1.getText().equals("-"))) {
					tabThiefThiefValue2.setText(tabThiefThiefValue1.getText());
					tabThiefThief1.setText(parseMyText(Dictionary.getText("THIEF3"), tabThiefThiefValue1.getText()));
					tabThiefThief2.setText(parseMyText(Dictionary.getText("THIEF3"), tabThiefThiefValue1.getText()));
				}
				if (rdbtnMutateThief.isSelected())
					calculateMutateThiefBonus(mutateThiefValue.getText());
			}
		});

		/**
		 * Obsuga pola w macierzy wypat.
		 */
		tabThiefThiefValue2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (tabThiefThiefValue2.getText().equals("-") || tabThiefThiefValue2.getText().equals("."))
					tabThiefThiefValue2.setText("0.0");
				if (tabThiefThiefValue2.getText().length() >= 8)
					tabThiefThiefValue2.setText("200.0");
				if (!tabThiefThiefValue2.getText().isEmpty() && !(tabThiefThiefValue2.getText().equals("-"))) {
					tabThiefThiefValue1.setText(tabThiefThiefValue2.getText());
					tabThiefThief1.setText(parseMyText(Dictionary.getText("THIEF3"), tabThiefThiefValue2.getText()));
					tabThiefThief2.setText(parseMyText(Dictionary.getText("THIEF3"), tabThiefThiefValue2.getText()));
				}
				if (rdbtnMutateThief.isSelected())
					calculateMutateThiefBonus(mutateThiefValue.getText());

			}
		});

		/**
		 * Obsuga pola w macierzy wypat.
		 */
		tabThiefProducerValue2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (tabThiefProducerValue2.getText().equals("-") || tabThiefProducerValue2.getText().equals("."))
					tabThiefProducerValue2.setText("0.0");
				if (tabThiefProducerValue2.getText().length() >= 8)
					tabThiefProducerValue2.setText("200.0");
				if (!tabThiefProducerValue2.getText().isEmpty() && !(tabThiefProducerValue2.getText().equals("-"))) {
					tabThiefProducerValue1.setText(tabThiefProducerValue2.getText());
					tabThiefProducer1.setText(parseMyText(Dictionary.getText("THIEF3"), tabThiefProducerValue2.getText()));
					tabThiefProducer2.setText(parseMyText(Dictionary.getText("THIEF3"), tabThiefProducerValue2.getText()));
				}
				if (rdbtnMutateThief.isSelected())
					calculateMutateThiefBonus(mutateThiefValue.getText());
			}
		});

		/**
		 * Obsuga pola w macierzy wypat.
		 */
		tabProducerThiefValue2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (tabProducerThiefValue2.getText().equals("-") || tabProducerThiefValue2.getText().equals("."))
					tabProducerThiefValue2.setText("0.0");
				if (tabProducerThiefValue2.getText().length() >= 8)
					tabProducerThiefValue2.setText("200.0");
				if (!tabProducerThiefValue2.getText().isEmpty() && !(tabProducerThiefValue2.getText().equals("-"))) {
					tabProducerThiefValue1.setText(tabProducerThiefValue2.getText());
					tabProducerThief1.setText(parseMyText(Dictionary.getText("PRODUCER3"), tabProducerThiefValue2.getText()));
					tabProducerThief2.setText(parseMyText(Dictionary.getText("PRODUCER3"), tabProducerThiefValue2.getText()));
				}
				if (rdbtnMutateProducer.isSelected())
					calculateMutateProducerBonus(mutateProducerValue.getText());
			}
		});

		/**
		 * Obsuga pola w macierzy wypat.
		 */
		tabProducerProducerValue1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (tabProducerProducerValue1.getText().equals("-") || tabProducerProducerValue1.getText().equals("."))
					tabProducerProducerValue1.setText("0.0");
				if (tabProducerProducerValue1.getText().length() >= 8)
					tabProducerProducerValue1.setText("200.0");
				if (!tabProducerProducerValue1.getText().isEmpty() && !(tabProducerProducerValue1.getText().equals("-"))) {
					tabProducerProducerValue2.setText(tabProducerProducerValue1.getText());
					tabProducerProducer1
							.setText(parseMyText(Dictionary.getText("PRODUCER3"), tabProducerProducerValue1.getText()));
					tabProducerProducer2
							.setText(parseMyText(Dictionary.getText("PRODUCER3"), tabProducerProducerValue1.getText()));
				}
				if (rdbtnMutateProducer.isSelected())
					calculateMutateProducerBonus(mutateProducerValue.getText());
			}
		});

		/**
		 * Obsuga pola w macierzy wypat.
		 */
		tabProducerProducerValue2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (tabProducerProducerValue2.getText().equals("-") || tabProducerProducerValue2.getText().equals("."))
					tabProducerProducerValue2.setText("0.0");
				if (tabProducerProducerValue2.getText().length() >= 8)
					tabProducerProducerValue2.setText("200.0");
				if (!tabProducerProducerValue2.getText().isEmpty() && !(tabProducerProducerValue2.getText().equals("-"))) {
					tabProducerProducerValue1.setText(tabProducerProducerValue2.getText());
					tabProducerProducer1
							.setText(parseMyText(Dictionary.getText("PRODUCER3"), tabProducerProducerValue2.getText()));
					tabProducerProducer2
							.setText(parseMyText(Dictionary.getText("PRODUCER3"), tabProducerProducerValue2.getText()));
				}
				if (rdbtnMutateProducer.isSelected())
					calculateMutateProducerBonus(mutateProducerValue.getText());
			}
		});

		/**
		 * Obsuga pola w macierzy wypat.
		 */
		tabProducerThiefValue1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (tabProducerThiefValue1.getText().equals("-") || tabProducerThiefValue1.getText().equals("."))
					tabProducerThiefValue1.setText("0.0");
				if (tabProducerThiefValue1.getText().length() >= 8)
					tabProducerThiefValue1.setText("200.0");
				if (!tabProducerThiefValue1.getText().isEmpty() && !(tabProducerThiefValue1.getText().equals("-"))) {
					if (!tabProducerThiefValue1.getText().isEmpty()) {
						if (!tabProducerThiefValue1.getText().substring(0, 1).matches("[-0-9]"))
							tabProducerThiefValue1.setText("0.0");
						tabProducerThiefValue2.setText(tabProducerThiefValue1.getText());
						tabProducerThief2
								.setText(parseMyText(Dictionary.getText("PRODUCER3"), tabProducerThiefValue1.getText()));
						tabProducerThief1
								.setText(parseMyText(Dictionary.getText("PRODUCER3"), tabProducerThiefValue1.getText()));
					}
				}
				if (rdbtnMutateProducer.isSelected())
					calculateMutateProducerBonus(mutateProducerValue.getText());
			}
		});

		/**
		 * Obsuga pola w macierzy wypat.
		 */
		tabThiefProducerValue1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (tabThiefProducerValue1.getText().equals("-") || tabThiefProducerValue1.getText().equals("."))
					tabThiefProducerValue1.setText("0.0");
				if (tabThiefProducerValue1.getText().length() >= 8)
					tabThiefProducerValue1.setText("200.0");
				if (!tabThiefProducerValue1.getText().isEmpty() && !(tabThiefProducerValue1.getText().equals("-"))) {
					tabThiefProducerValue2.setText(tabThiefProducerValue1.getText());
					tabThiefProducer2.setText(parseMyText(Dictionary.getText("THIEF3"), tabThiefProducerValue1.getText()));
					tabThiefProducer1.setText(parseMyText(Dictionary.getText("THIEF3"), tabThiefProducerValue1.getText()));
				}
				if (rdbtnMutateThief.isSelected())
					calculateMutateThiefBonus(mutateThiefValue.getText());

			}
		});

		/**
		 * Obsuga paska liczebnoci zodziei.
		 */
		thiefsScrollBar.addAdjustmentListener(new AdjustmentListener() {
			@Override
			public void adjustmentValueChanged(AdjustmentEvent e) {
				thiefsAmount.setText(String.valueOf(thiefsScrollBar.getValue()));
				populationAmount.setText(String.valueOf(thiefsScrollBar.getValue() + producersScrollBar.getValue()));
			}
		});

		/**
		 * Obsuga paska liczebnoci producentw.
		 */
		producersScrollBar.addAdjustmentListener(new AdjustmentListener() {
			@Override
			public void adjustmentValueChanged(AdjustmentEvent e) {

				producersAmount.setText(String.valueOf(producersScrollBar.getValue()));
				populationAmount.setText(String.valueOf(thiefsScrollBar.getValue() + producersScrollBar.getValue()));
			}
		});

		/**
		 * Obsuga liczby zodziei.
		 */
		thiefsAmount.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (thiefsAmount.getText().length() >= 5)
					thiefsAmount.setText("4999");
				if (!thiefsAmount.getText().isEmpty()) {
					thiefsScrollBar.setValue(Integer.valueOf(thiefsAmount.getText()));
					populationAmount.setText(String.valueOf(
							Integer.valueOf(thiefsAmount.getText()) + Integer.valueOf(producersAmount.getText())));
					if (thiefsScrollBar.getMaximum() < Integer.valueOf(thiefsAmount.getText())) {
						thiefsScrollBar.setMaximum(Integer.valueOf(thiefsAmount.getText()));
						thiefsScrollBar.setValue(Integer.valueOf(thiefsAmount.getText()));
					}
				}
			}
		});

		/**
		 * Obsuga liczby producentw.
		 */
		producersAmount.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (producersAmount.getText().length() >= 5)
					producersAmount.setText("5000");
				if (!producersAmount.getText().isEmpty()) {
					producersScrollBar.setValue(Integer.valueOf(producersAmount.getText()));
					populationAmount.setText(String.valueOf(
							Integer.valueOf(thiefsAmount.getText()) + Integer.valueOf(producersAmount.getText())));
					if (producersScrollBar.getMaximum() < Integer.valueOf(producersAmount.getText())) {
						producersScrollBar.setMaximum(Integer.valueOf(producersAmount.getText()));
						producersScrollBar.setValue(Integer.valueOf(producersAmount.getText()));
					}
				}
			}
		});

		/**
		 * Obsuga liczebnoci populacji.
		 */
		populationAmount.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (populationAmount.getText().length() >= 5)
					populationAmount.setText("9999");
				if (!populationAmount.getText().isEmpty()) {
					Double tmp = Math.floor((Double.valueOf(populationAmount.getText()) / 2));
					int parser = tmp.intValue();
					thiefsAmount.setText(String.valueOf(parser));

					thiefsScrollBar.setMaximum(parser + 1);
					thiefsScrollBar.setValue(parser);
					repaint();

					tmp = Math.ceil((Double.valueOf(populationAmount.getText()) / 2));
					parser = tmp.intValue();
					producersAmount.setText(String.valueOf(parser));

					producersScrollBar.setMaximum(parser + 1);
					producersScrollBar.setValue(parser);
					repaint();
				}
			}
		});

		/**
		 * Obsuga fokusu.
		 */
		contentPane.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				super.mouseReleased(e);
				MainWindow.this.getContentPane().grabFocus();
			}
		});

		this.addWindowListener(new WindowEventHandler());
	}

	/**
	 * Przygotowanie pl obsugi dguoci cyklu
	 */
	private void prepareLengthOfCycleSpinners()
	{
		lengthOfCycleFrom.setModel(new SpinnerNumberModel(1 ,ViewConsts.MIN_CYCLE_LENGTH ,ViewConsts.MAX_CYCLE_LENGTH , 1));
		lengthOfCycleTo.setModel(new SpinnerNumberModel(1000 ,ViewConsts.MIN_CYCLE_LENGTH ,ViewConsts.MAX_CYCLE_LENGTH , 1));

		JFormattedTextField txtFrom = ((JSpinner.NumberEditor) lengthOfCycleFrom.getEditor()).getTextField();
		((NumberFormatter) txtFrom.getFormatter()).setAllowsInvalid(false);
		JFormattedTextField txtTo = ((JSpinner.NumberEditor) lengthOfCycleTo.getEditor()).getTextField();
		((NumberFormatter) txtTo.getFormatter()).setAllowsInvalid(false);
	}

	/**
	 * Obsuga wpisiywania liczb zmiennoprzecinkowych.
	 * @param txtField
	 * @param minValue
	 * @param maxValue
	 */
	public void doubleAmountWrapper(JTextField txtField, Double minValue, Double maxValue) {
		txtField.addKeyListener(new KeyAdapter() {
			public void keyTyped(KeyEvent e) {

				char c = e.getKeyChar();
				if (!((c >= '0') && (c <= '9') || (c == KeyEvent.VK_BACK_SPACE) || (c == KeyEvent.VK_DELETE)
						|| (c == KeyEvent.VK_MINUS) || (c == KeyEvent.VK_PERIOD))) {
					getToolkit().beep();
					e.consume();
				}
				if (c == KeyEvent.VK_MINUS && minValue.compareTo(0d) > 0) {
					getToolkit().beep();
					e.consume();
				}

				if (c == KeyEvent.VK_MINUS && txtField.getCaretPosition() != 0) {
					getToolkit().beep();
					e.consume();
				}

				if (c == KeyEvent.VK_PERIOD && (txtField.getCaretPosition() == 0
						|| (txtField.getCaretPosition() == 1 && txtField.getText().startsWith("-")))) {
					getToolkit().beep();
					e.consume();
				}
			}

			@Override
			public void keyReleased(KeyEvent e) {

				char c = e.getKeyChar();
				if (c == KeyEvent.VK_MINUS && txtField.getText().length() == 1)
					return;
				if (c == KeyEvent.VK_BACK_SPACE)
					return;
				if (txtField.getText().replace(".", "").length() < txtField.getText().length() - 1) {
					int index = txtField.getText().indexOf(".");
					String tmp = txtField.getText().substring(0, index + 1);
					txtField.setText(tmp);
				}
				if (txtField.getText().replace("-", "").length() < txtField.getText().length() - 1) {
					int index = txtField.getText().indexOf("-");
					String tmp = txtField.getText().substring(0, index + 1);
					txtField.setText(tmp);
				}
				if (!txtField.getText().isEmpty() && !txtField.getText().startsWith(".")
						&& !(txtField.getText().startsWith("-")) && Double.valueOf(txtField.getText()) >= maxValue) {
					txtField.setText(String.format("%.2f", maxValue).replace(',', '.'));
				}
				if (!txtField.getText().isEmpty() && txtField.getText().startsWith("-")
						&& txtField.getText().length() > 1 && Double.valueOf(txtField.getText()) <= minValue) {
					System.out.println(String.format("%.2f", minValue).replace(',', '.'));
					txtField.setText(String.format("%.2f", minValue).replace(',', '.'));
				}

				if (!txtField.getText().isEmpty() && !DOUBLE_PATTERN.matcher(txtField.getText()).matches())
					txtField.setText(String.format("%.2f", minValue).replace(',', '.'));

				txtField.postActionEvent();

			}

			@Override
			public void keyPressed(KeyEvent e) {
				if (txtField.getText().length() >= 6) {
					e.consume();
					txtField.setText(String.valueOf(maxValue));
				}
				if (txtField.getText().endsWith(".."))
					txtField.setText(txtField.getText().substring(0, txtField.getText().length() - 1));
			}
		});
		txtField.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent evt) {
				txtField.selectAll();
			}

			public void focusLost(FocusEvent e) {
				if (txtField.getText().isEmpty())
					txtField.setText(String.format("%.2f", minValue).replace(',', '.'));
				if (txtField.getText().replace(".", "").length() < txtField.getText().length() - 1) {
					int index = txtField.getText().indexOf(".");
					String tmp = txtField.getText().substring(index + 1);
					tmp = tmp.replace(".", "");
					txtField.setText(tmp);
				}
				if (txtField.getText().endsWith("."))
					txtField.setText(txtField.getText() + "0");
				if (txtField.getText().startsWith("."))
					txtField.setText("0" + txtField.getText());
				txtField.postActionEvent();
			}
		});

	}

	/**
	 * Obsuga wpisywania liczb naturalnych.
	 * @param txtField
	 * @param minValue
	 * @param maxValue
	 */
	public void integerAmountWrapper(JTextField txtField, int minValue, int maxValue) {
		txtField.addKeyListener(new KeyAdapter() {
			public void keyTyped(KeyEvent e) {
				char c = e.getKeyChar();
				if (!((c >= '0') && (c <= '9') || (c == KeyEvent.VK_BACK_SPACE) || (c == KeyEvent.VK_DELETE))) {
					getToolkit().beep();
					e.consume();
				}
			}
			public void keyReleased(KeyEvent e) {
				if (!txtField.getText().isEmpty() && Integer.valueOf(txtField.getText()) >= maxValue)
					txtField.setText(String.valueOf(maxValue));
				if (txtField.getText().startsWith("0"))
					txtField.setText("0");
				txtField.postActionEvent();
			}
			public void keyPressed(KeyEvent e) {
				if (txtField.getText().length() >= 6) {
					e.consume();
					txtField.setText(String.valueOf(maxValue));
				}
			}
		});
		txtField.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent evt) {
				txtField.selectAll();
			}

			public void focusLost(FocusEvent e) {
				if (txtField.getText().isEmpty())
					txtField.setText(String.valueOf(minValue));
				if (txtField.getText().startsWith("0"))
					txtField.setText(String.valueOf(minValue));
				txtField.postActionEvent();
			}
		});
	}

	/**
	 * Wczytanie ustawie.
	 */
	private void readProperties() {
		String propFileName = ViewConsts.FILE_PROPERTIES_NAME;
		try (FileReader reader = new FileReader(propFileName);) {
			Properties prop = new Properties();
			prop.load(reader);

			thiefsAmount.setText(prop.getProperty("THIEFSAMOUNT"));
			producersAmount.setText(prop.getProperty("PRODUCERSAMOUNT"));
			populationAmount.setText(
					String.valueOf(Integer.valueOf(thiefsAmount.getText()) + Integer.valueOf(producersAmount.getText())));

			if (producersScrollBar.getMaximum() < Integer.valueOf(producersAmount.getText()))
				producersScrollBar.setMaximum(Integer.valueOf(producersAmount.getText()));
			if (thiefsScrollBar.getMaximum() < Integer.valueOf(thiefsAmount.getText()))
				thiefsScrollBar.setMaximum(Integer.valueOf(thiefsAmount.getText()));
			thiefsScrollBar.setValue(Integer.valueOf(prop.getProperty("THIEFSAMOUNT")));
			producersScrollBar.setValue(Integer.valueOf(prop.getProperty("PRODUCERSAMOUNT")));
			mutateThiefValue.setText(prop.getProperty("THIEFS_MUTATE_PERCENT"));
			mutateProducerValue.setText(prop.getProperty("PRODUCERS_MUTATE_PERCENT"));
			if (Boolean.valueOf(prop.getProperty("PERCENT_THIEF_MUTATE_BOOL"))) {
				rdbtnMutateThief.setSelected(true);
				mutateThiefValue.setEnabled(true);
				setVisibleMutateThiefValues(true);
			}
			if (Boolean.valueOf(prop.getProperty("PERCENT_PRODUCER_MUTATE_BOOL"))) {
				rdbtnMutateProducer.setSelected(true);
				mutateProducerValue.setEnabled(true);
				setVisibleMutateProducerValues(true);
			}
			if (Boolean.valueOf(prop.getProperty("MANUALLY_THIEF_MUTATE_BOOL"))) {
				rdbtnMutateThiefWithTextFields.setSelected(true);
				setVisibleMutateThiefValues(true);
				setEnabledMutateThiefValues(true);
			}
			if (Boolean.valueOf(prop.getProperty("MANUALLY_PRODUCER_MUTATE_BOOL"))) {
				rdbtnMutateProducerWithTextFields.setSelected(true);
				setVisibleMutateProducerValues(true);
				setEnabledMutateProducerValues(true);
			}

			lengthOfCycleFrom.setValue(Integer.valueOf(prop.getProperty("ROUND_FROM")));
			lengthOfCycleTo.setValue(Integer.valueOf(prop.getProperty("ROUND_TO")));

		} catch (Exception e) {
			System.err.println("Exception: " + e);
		}
	}

	/**
	 * Stworzenie domylnych ustawie
	 * @throws IOException
	 */
	private void makeDefaultProperties() throws IOException {
		deleteProperties();

		File file = new File(ViewConsts.FILE_PROPERTIES_NAME);
		String defaultData = null;

		file.createNewFile();
		defaultData = "THIEFSAMOUNT=" + thiefsAmount.getText() + System.getProperty("line.separator");
		defaultData += "PRODUCERSAMOUNT=" + producersAmount.getText() + System.getProperty("line.separator");
		defaultData += "THIEFS_MUTATE_PERCENT=" + mutateThiefValue.getText() + System.getProperty("line.separator");
		defaultData += "PRODUCERS_MUTATE_PERCENT=" + mutateProducerValue.getText() + System.getProperty("line.separator");
		defaultData += "PERCENT_THIEF_MUTATE_BOOL=" + rdbtnMutateThief.isSelected() + System.getProperty("line.separator");
		defaultData += "PERCENT_PRODUCER_MUTATE_BOOL=" + rdbtnMutateProducer.isSelected() + System.getProperty("line.separator");
		defaultData += "MANUALLY_THIEF_MUTATE_BOOL=" + rdbtnMutateThiefWithTextFields.isSelected() + System.getProperty("line.separator");
		defaultData += "MANUALLY_PRODUCER_MUTATE_BOOL=" + rdbtnMutateProducerWithTextFields.isSelected() + System.getProperty("line.separator");
		defaultData += "ROUND_FROM=" + lengthOfCycleFrom.getValue() + System.getProperty("line.separator");
		defaultData += "ROUND_TO=" + lengthOfCycleTo.getValue();
		FileWriter fileWritter = new FileWriter(file.getName(), true);
		BufferedWriter bufferWritter = new BufferedWriter(fileWritter);
		bufferWritter.write(defaultData);
		bufferWritter.close();
	}

	/**
	 * Usunicie ustawie
	 */
	private void deleteProperties() {
		File file = new File(ViewConsts.FILE_PROPERTIES_NAME);
		file.delete();
	}

	/**
	 * Przygotowanie tekstw do macierzy wypat.
	 * @param who
	 * @param value
	 * @return
	 */
	private String parseMyText(String who, String value) {
		Double i = Double.valueOf(value);
		if (i == 0)
			return who + Dictionary.getText("STRING_PARSING_NOT_EARN");
		else if (i < 0)
			return who + Dictionary.getText("STRING_PARSING_LOOSE") + String.valueOf(i * (-1));
		else
			return who + Dictionary.getText("STRING_PARSING_EARN") + i;
	}

	/**
	 * Oblicz bonus zmutowanego zodzieja.
	 * @param thiefBonus
	 */
	private void calculateMutateThiefBonus(String thiefBonus) {
		try {
			if (tabThiefThiefValue1.getText().equals("-") || tabThiefProducerValue1.getText().equals("-"))
				return;

			if (Double.valueOf(tabThiefThiefValue1.getText()) >= 0)
				tabMutateThiefThiefValue.setText(String.format("%.2f",
						(1.0 + Double.valueOf(thiefBonus) * 0.01) * Double.valueOf(tabThiefThiefValue1.getText()))
						.replace(',', '.'));
			else
				tabMutateThiefThiefValue.setText(String.format("%.2f",
						(1.0 - Double.valueOf(thiefBonus) * 0.01) * Double.valueOf(tabThiefThiefValue1.getText()))
						.replace(',', '.'));
			if (Double.valueOf(tabThiefProducerValue1.getText()) >= 0)
				tabMutateThiefProducerValue.setText(String.format("%.2f",
						(1.0 + Double.valueOf(thiefBonus) * 0.01) * Double.valueOf(tabThiefProducerValue1.getText()))
						.replace(',', '.'));
			else
				tabMutateThiefProducerValue.setText(String.format("%.2f",
						(1.0 - Double.valueOf(thiefBonus) * 0.01) * Double.valueOf(tabThiefProducerValue1.getText()))
						.replace(',', '.'));
		} catch (NumberFormatException ex) {
			mutateThiefValue.setText("1");
			calculateMutateThiefBonus("1");
		}
	}

	/**
	 * Oblicz bonus zmutowanego producenta.
	 * @param producerBonus
	 */
	private void calculateMutateProducerBonus(String producerBonus) {
		try {
			if (tabProducerProducerValue1.getText().equals("-") || tabProducerThiefValue2.getText().equals("-"))
				return;

			if (Double.valueOf(tabProducerProducerValue1.getText()) >= 0)
				tabMutateProducerProducerValue.setText(String.format("%.2f",
						(1.0 + Double.valueOf(producerBonus) * 0.01) * Double.valueOf(tabProducerProducerValue1.getText()))
						.replace(',', '.'));
			else
				tabMutateProducerProducerValue.setText(String.format("%.2f",
						(1.0 - Double.valueOf(producerBonus) * 0.01) * Double.valueOf(tabProducerProducerValue1.getText()))
						.replace(',', '.'));
			if (Double.valueOf(tabProducerThiefValue2.getText()) >= 0)
				tabMutateProducerThiefValue.setText(String.format("%.2f",
						(1.0 + Double.valueOf(producerBonus) * 0.01) * Double.valueOf(tabProducerThiefValue2.getText()))
						.replace(',', '.'));
			else
				tabMutateProducerThiefValue.setText(String.format("%.2f",
						(1.0 - Double.valueOf(producerBonus) * 0.01) * Double.valueOf(tabProducerThiefValue2.getText()))
						.replace(',', '.'));
		} catch (NumberFormatException ex) {
			mutateProducerValue.setText("1");
			calculateMutateProducerBonus("1");
		}
	}

	/**
	 * Zastosuj ustawienia domylne.
	 */
	private void setDefaultValues() {
		tabThiefThiefValue1.setText(String.valueOf(ViewConsts.DEF_VALUE_THIEFTHIEF));
		tabThiefThiefValue2.setText(String.valueOf(ViewConsts.DEF_VALUE_THIEFTHIEF));
		tabProducerThiefValue1.setText(String.valueOf(ViewConsts.DEF_VALUE_PRODUCERTHIEF));
		tabThiefProducerValue1.setText(String.valueOf(ViewConsts.DEF_VALUE_THIEFPRODUCER));
		tabThiefProducerValue2.setText(String.valueOf(ViewConsts.DEF_VALUE_THIEFPRODUCER));
		tabProducerThiefValue2.setText(String.valueOf(ViewConsts.DEF_VALUE_PRODUCERTHIEF));
		tabProducerProducerValue1.setText(String.valueOf(ViewConsts.DEF_VALUE_PRODUCERPRODUCER));
		tabProducerProducerValue2.setText(String.valueOf(ViewConsts.DEF_VALUE_PRODUCERPRODUCER));
		mutateThiefValue.setText(String.valueOf(ViewConsts.DEF_VALUE_PERCENT_MUTATION));
		mutateProducerValue.setText(String.valueOf(ViewConsts.DEF_VALUE_PERCENT_MUTATION));
		calculateMutateThiefBonus(String.valueOf(ViewConsts.DEF_VALUE_PERCENT_MUTATION));
		calculateMutateProducerBonus(String.valueOf(ViewConsts.DEF_VALUE_PERCENT_MUTATION));
		tabThiefThiefValue1.postActionEvent();
		tabThiefProducerValue1.postActionEvent();
		tabProducerThiefValue1.postActionEvent();
		tabProducerProducerValue1.postActionEvent();
		rdbtnMutateThief.setSelected(false);
		rdbtnMutateThiefWithTextFields.setSelected(false);
		rdbtnMutateProducer.setSelected(false);
		rdbtnMutateProducerWithTextFields.setSelected(false);
		setVisibleMutateProducerValues(false);
		setVisibleMutateThiefValues(false);
	}

	private void setVisibleMutateThiefValues(boolean b) {
		tabMutateThiefThiefValue.setVisible(b);
		tabMutateThiefProducerValue.setVisible(b);
		txtMutatedThief.setVisible(b);
	}

	private void setVisibleMutateProducerValues(boolean b) {
		tabMutateProducerThiefValue.setVisible(b);
		tabMutateProducerProducerValue.setVisible(b);
		txtMutatedProducer.setVisible(b);
	}

	private void setEnabledMutateThiefValues(boolean b) {
		tabMutateThiefThiefValue.setEnabled(b);
		tabMutateThiefProducerValue.setEnabled(b);
	}

	private void setEnabledMutateProducerValues(boolean b) {
		tabMutateProducerThiefValue.setEnabled(b);
		tabMutateProducerProducerValue.setEnabled(b);
	}

	/**
	 * Obsuga wyjcia z programu.
	 */
	class WindowEventHandler extends WindowAdapter {
		@SuppressWarnings("static-access")
		public void windowClosing(WindowEvent ev) {
			if (!Settings.isLog()) {
				setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				try {
					makeDefaultProperties();
				} catch (IOException e) {
					e.printStackTrace();
				}
				return;
			}

			Object[] options = { Dictionary.getText("YES"), Dictionary.getText("NO") };

			UIManager UI = new UIManager();
			UI.put("OptionPane.background", Settings.getColor("BACKGROUND"));
			UI.put("Panel.background", Settings.getColor("BACKGROUND"));

			int dialogResult = JOptionPane.showOptionDialog(MainWindow.this.getContentPane(),
					Dictionary.getText("EXIT_QUESTION"), Dictionary.getText("REPORT"), 0,
					JOptionPane.INFORMATION_MESSAGE, null, options, null);
			if (dialogResult == 0)
				System.out.println("Yes option");
			else
				while (dirCounter > 0) {
					File dir = new File(Params.DIR_NAME + dirCounter--);
					Simulation.deleteDir(dir);
					dir.delete();
				}

			if (dialogResult == 0 || dialogResult == 1)
				setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			else
				return;
			try {
				makeDefaultProperties();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public JLabel getStrengthSelection() {
		return strengthSelection;
	}

	public JPanel getContentPane() {
		return contentPane;
	}

	public Integer getLengthOfCycleFrom() {
		return (Integer) lengthOfCycleFrom.getValue();
	}

	public Integer getLengthOfCycleTo() {
		return (Integer) lengthOfCycleTo.getValue();
	}
}
