/**
 * Maa ewolucja - Gra ewolucyjno spoeczna
 *
 *	Okno symulacji.
 *
 * @author Grzegorz Przytua, Adam Makowski
 */

package view;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import logic.PopulationMember;
import logic.Simulation;
import settings.Dictionary;
import settings.Settings;
import javax.swing.JSpinner;
import javax.swing.JRadioButton;

@SuppressWarnings("serial")
public class SimulationWindow extends JFrame
{
	/**
	 * Odniesienie do symulacji.
	 */
	private Simulation countingThread;
	
	/**
	 * Parametry symulacji
	 */
	private int thiefsAmount;
	private int producersAmount;
	private int strengthSelection;
	private int frequencyFrom;
	private int frequencyTo;
	int populationAmount;
	
	private boolean thiefMutate;
	private boolean producerMutate;
	
	private Double thiefVsThief;
	private Double producerVsProducer;
	private Double thiefVsProducer;
	private Double producerVsThief;
	private Double mutateThiefVsThief;
	private Double mutateProducerVsProducer;
	private Double mutateThiefVsProducer;
	private Double mutateProducerVsThief;
	
	/**
	 * Elementy GUI
	 */
	private JPanel contentPane;

	private JProgressBar thiefsProgressBar;
	private JProgressBar producersProgressBar;
	private JProgressBar mutatedThiefsProgressBar;
	private JProgressBar mutatedProducersProgressBar;
	
	private JTextField textFieldThief;
	private JTextField textFieldProducer;
	private JTextField textFieldMutatedThief;
	private JTextField textFieldMutatedProducer;
	private JTextField delayTextField;
	private JTextField txtStartThiefsValue;
	private JTextField txtStartProducersValue;
	private JTextField txtThiefs;
	private JTextField txtProducers;
	private JTextField txtMutatedThiefs;
	private JTextField txtMutatedProducers;

	private JTextPane thiefsMutationAmountInfo;
	private JTextPane producersMutationAmountInfo;
	private JTextPane txtDelay;

	private PopulationProportionPane populationPercentPane;

	private JSpinner width;
	
	private JLabel widthLbl;
	private JLabel lblProportionGraph;
	private JLabel lblBaseInfo;
	private JLabel lblTitle;
	private JLabel lblMs;
	private JLabel lblLengthOfCycle;

	private JButton btnMutate;
	private JButton btnStop;
	private JButton btnStart;
	private JButton btnReset;
	private JButton btnReport;
	
	private JRadioButton rdbtnProportionMode;
	private JRadioButton rdbtnNormalMode;
	private ButtonGroup modeChooser;
	
	private Chart chartMeanPanel;
	private Chart chartPanelThief;
	private Chart chartPanelProducer;
	
	public SimulationWindow()
	{
		this(0, 0, 0d, 0d, 0d, 0d, 0, 0, 0, true, true);
	}

	/**
	 * Konstruktor.
	 * @param thiefsAmount
	 * @param producersAmount
	 * @param thiefVsThief
	 * @param producerVsProducer
	 * @param thiefVsProducer
	 * @param producerVsThief
	 * @param strengthSelection
	 * @param frequencyFrom
	 * @param frequencyTo
	 * @param mutatedThiefsOn
	 * @param mutatedProducersOn
	 */
	public SimulationWindow(int thiefsAmount, int producersAmount, Double thiefVsThief, Double producerVsProducer,
			Double thiefVsProducer, Double producerVsThief, int  strengthSelection, int frequencyFrom, int frequencyTo, boolean mutatedThiefsOn, boolean mutatedProducersOn)
	{
		super(Dictionary.getText(Dictionary.getText("SIMULATION") + " (" + (MainWindow.dirCounter+1) + ")"));
		
		MainWindow.dirCounter++;
		
		mutateThiefVsThief = 0.0;
		mutateThiefVsProducer = 0.0;
		mutateProducerVsThief = 0.0;
		mutateProducerVsProducer = 0.0;
		
		this.populationAmount = thiefsAmount + producersAmount;
		this.thiefsAmount = thiefsAmount;
		this.producersAmount = producersAmount;
		this.thiefVsThief = thiefVsThief;
		this.producerVsProducer = producerVsProducer;
		this.thiefVsProducer = thiefVsProducer;
		this.producerVsThief = producerVsThief;
		this.strengthSelection = strengthSelection;
		this.frequencyFrom = frequencyFrom;
		this.frequencyTo = frequencyTo;
		thiefMutate = false;
		producerMutate = false;

		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setBounds(100, 100, 1679, 1000);
		setResizable(false);
		contentPane = new JPanel();
		contentPane.setBackground(Settings.getColor("BACKGROUND_SIMULATION"));
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		contentPane.setLayout(null);
		setContentPane(contentPane);

		ImageIcon img = new ImageIcon(ViewConsts.ICON_PATH);
		setIconImage(img.getImage());

		thiefsProgressBar = new JProgressBar();
		thiefsProgressBar.setBorderPainted(false);
		thiefsProgressBar.setBorder(null);
		thiefsProgressBar.setBackground(Settings.getColor("SCROLLS_SIMULATION"));
		thiefsProgressBar.setForeground(Settings.getColor("THIEF_COLOR_SIMULATION"));
		thiefsProgressBar.setBounds(146, 110, 358, 29);
		thiefsProgressBar.setStringPainted(true);
		thiefsProgressBar.setMaximum(populationAmount);
		thiefsProgressBar.setValue(thiefsAmount);
		contentPane.add(thiefsProgressBar);

		producersProgressBar = new JProgressBar();
		producersProgressBar.setBorderPainted(false);
		producersProgressBar.setBorder(null);
		producersProgressBar.setBackground(Settings.getColor("SCROLLS_SIMULATION"));
		producersProgressBar.setForeground(Settings.getColor("PRODUCER_COLOR_SIMULATION"));
		producersProgressBar.setStringPainted(true);
		producersProgressBar.setBounds(146, 177, 358, 29);
		producersProgressBar.setMaximum(populationAmount);
		producersProgressBar.setValue(producersAmount);
		contentPane.add(producersProgressBar);

		mutatedThiefsProgressBar = new JProgressBar();
		mutatedThiefsProgressBar.setBorderPainted(false);
		mutatedThiefsProgressBar.setBorder(null);
		mutatedThiefsProgressBar.setBackground(Settings.getColor("SCROLLS_SIMULATION"));
		mutatedThiefsProgressBar.setForeground(Settings.getColor("MUTATED_THIEF_COLOR_SIMULATION"));
		mutatedThiefsProgressBar.setStringPainted(true);
		mutatedThiefsProgressBar.setBounds(146, 244, 358, 29);
		mutatedThiefsProgressBar.setMaximum(populationAmount);
		contentPane.add(mutatedThiefsProgressBar);

		mutatedProducersProgressBar = new JProgressBar();
		mutatedProducersProgressBar.setBorderPainted(false);
		mutatedProducersProgressBar.setBorder(null);
		mutatedProducersProgressBar.setBackground(Settings.getColor("SCROLLS_SIMULATION"));
		mutatedProducersProgressBar.setForeground(Settings.getColor("MUTATED_PRODUCER_COLOR_SIMULATION"));
		mutatedProducersProgressBar.setStringPainted(true);
		mutatedProducersProgressBar.setBounds(146, 314, 358, 29);
		mutatedProducersProgressBar.setMaximum(populationAmount);
		contentPane.add(mutatedProducersProgressBar);
		
		txtThiefs = new JTextField();
		txtThiefs.setBackground(Settings.getColor("FILEDS_WITH_TEXTS"));
		txtThiefs.setDisabledTextColor(Settings.getColor("SIMULATION_WINDOW_FONT"));
		txtThiefs.setEnabled(false);
		txtThiefs.setHorizontalAlignment(SwingConstants.CENTER);
		txtThiefs.setFont(new Font("Tahoma", Font.PLAIN, 17));
		txtThiefs.setText(Dictionary.getText("THIEFS"));
		txtThiefs.setBounds(20, 110, 116, 29);
		contentPane.add(txtThiefs);
		txtThiefs.setColumns(10);

		txtProducers = new JTextField();
		txtProducers.setBackground(Settings.getColor("FILEDS_WITH_TEXTS"));
		txtProducers.setDisabledTextColor(Settings.getColor("SIMULATION_WINDOW_FONT"));
		txtProducers.setEnabled(false);
		txtProducers.setFont(new Font("Tahoma", Font.PLAIN, 17));
		txtProducers.setHorizontalAlignment(SwingConstants.CENTER);
		txtProducers.setText(Dictionary.getText("PRODUCERS"));
		txtProducers.setColumns(10);
		txtProducers.setBounds(20, 177, 116, 29);
		contentPane.add(txtProducers);

		txtMutatedThiefs = new JTextField();
		txtMutatedThiefs.setBackground(Settings.getColor("FILEDS_WITH_TEXTS"));
		txtMutatedThiefs.setDisabledTextColor(Settings.getColor("SIMULATION_WINDOW_FONT"));
		txtMutatedThiefs.setEnabled(false);
		txtMutatedThiefs.setFont(new Font("Tahoma", Font.PLAIN, 12));
		txtMutatedThiefs.setHorizontalAlignment(SwingConstants.CENTER);
		txtMutatedThiefs.setText(Dictionary.getText("MUTATED_THIEFS"));
		txtMutatedThiefs.setColumns(10);
		txtMutatedThiefs.setBounds(20, 244, 116, 29);
		contentPane.add(txtMutatedThiefs);

		txtMutatedProducers = new JTextField();
		txtMutatedProducers.setBackground(Settings.getColor("FILEDS_WITH_TEXTS"));
		txtMutatedProducers.setDisabledTextColor(Settings.getColor("SIMULATION_WINDOW_FONT"));
		txtMutatedProducers.setEnabled(false);
		txtMutatedProducers.setFont(new Font("Tahoma", Font.PLAIN, 12));
		txtMutatedProducers.setText(Dictionary.getText("MUTATED_PRODUCERS"));
		txtMutatedProducers.setHorizontalAlignment(SwingConstants.CENTER);
		txtMutatedProducers.setColumns(10);
		txtMutatedProducers.setBounds(20, 314, 116, 29);
		contentPane.add(txtMutatedProducers);

		lblTitle = new JLabel(Dictionary.getText("TITLE_SIMULATION") + " (" + MainWindow.dirCounter + ")");
		lblTitle.setBorder(null);
		lblTitle.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		lblTitle.setHorizontalAlignment(SwingConstants.CENTER);
		lblTitle.setFont(new Font("Tahoma", Font.BOLD, 32));
		lblTitle.setBounds(10, 10, 1639, 47);
		contentPane.add(lblTitle);

		btnStop = new JButton(Dictionary.getText("STOP"));
		btnStop.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		btnStop.setBackground(Settings.getColor("BUTTON_STOP_COLOR"));
		btnStop.setFont(new Font("Tahoma", Font.PLAIN, 20));
		btnStop.setBounds(185, 427, 154, 29);
		btnStop.setEnabled(false);
		contentPane.add(btnStop);
		btnStop.setToolTipText("<html><p width=\"100\">"
				+ Dictionary.getText("STOP_BUTTON_INFO")
				+ "</p></html>");

		btnStart = new JButton(Dictionary.getText("START"));
		btnStart.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		btnStart.setBackground(Settings.getColor("BUTTON_START_COLOR"));
		btnStart.setFont(new Font("Tahoma", Font.PLAIN, 20));
		btnStart.setBounds(20, 427, 154, 29);
		contentPane.add(btnStart);
		btnStart.setToolTipText("<html><p width=\"100\">"
				+ Dictionary.getText("RESTART_BUTTON_INFO")
				+ "</p></html>");

		btnReset = new JButton(Dictionary.getText("RESET"));
		btnReset.setEnabled(false);
		btnReset.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		btnReset.setFont(new Font("Tahoma", Font.PLAIN, 20));
		btnReset.setBackground(Settings.getColor("BUTTON_RESET_COLOR"));
		btnReset.setBounds(350, 427, 154, 29);
		contentPane.add(btnReset);
		btnReset.setToolTipText("<html><p width=\"100\">"
				+ Dictionary.getText("RESET_BUTTON_INFO")
				+ "</p></html>");

		btnMutate = new JButton(Dictionary.getText("INTROUDCE_MUTATE"));
		btnMutate.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		btnMutate.setBackground(Settings.getColor("BUTTON_MUTATE_COLOR"));
		btnMutate.setFont(new Font("Tahoma", Font.PLAIN, 15));
		btnMutate.setBounds(267, 467, 237, 29);
		btnMutate.setToolTipText("<html><p width=\"100\">"
				+ Dictionary.getText("MUTATION_BUTTON_INFO")
				+ "</p></html>");
		contentPane.add(btnMutate);
		btnMutate.setEnabled(false);

		textFieldThief = new JTextField();
		textFieldThief.setBorder(null);
		textFieldThief.setBackground(Settings.getColor("FIELDS_WITH_NUMBERS"));
		textFieldThief.setDisabledTextColor(Settings.getColor("SIMULATION_WINDOW_FONT"));
		textFieldThief.setFont(new Font("Tahoma", Font.PLAIN, 20));
		textFieldThief.setEnabled(false);
		textFieldThief.setBounds(529, 110, 89, 29);
		textFieldThief.setText(thiefsAmount + "");
		contentPane.add(textFieldThief);
		textFieldThief.setColumns(10);

		textFieldProducer = new JTextField();
		textFieldProducer.setBorder(null);
		textFieldProducer.setBackground(Settings.getColor("FIELDS_WITH_NUMBERS"));
		textFieldProducer.setDisabledTextColor(Settings.getColor("SIMULATION_WINDOW_FONT"));
		textFieldProducer.setFont(new Font("Tahoma", Font.PLAIN, 20));
		textFieldProducer.setEnabled(false);
		textFieldProducer.setColumns(10);
		textFieldProducer.setBounds(529, 177, 89, 29);
		textFieldProducer.setText(producersAmount+"");
		contentPane.add(textFieldProducer);

		textFieldMutatedThief = new JTextField();
		textFieldMutatedThief.setBorder(null);
		textFieldMutatedThief.setBackground(Settings.getColor("FIELDS_WITH_NUMBERS"));
		textFieldMutatedThief.setDisabledTextColor(Settings.getColor("SIMULATION_WINDOW_FONT"));
		textFieldMutatedThief.setText("0");
		textFieldMutatedThief.setEnabled(false);
		textFieldMutatedThief.setFont(new Font("Tahoma", Font.PLAIN, 20));
		textFieldMutatedThief.setColumns(10);
		textFieldMutatedThief.setBounds(529, 244, 89, 29);
		contentPane.add(textFieldMutatedThief);

		textFieldMutatedProducer = new JTextField();
		textFieldMutatedProducer.setBorder(null);
		textFieldMutatedProducer.setBackground(Settings.getColor("FIELDS_WITH_NUMBERS"));
		textFieldMutatedProducer.setDisabledTextColor(Settings.getColor("SIMULATION_WINDOW_FONT"));
		textFieldMutatedProducer.setText("0");
		textFieldMutatedProducer.setEnabled(false);
		textFieldMutatedProducer.setFont(new Font("Tahoma", Font.PLAIN, 20));
		textFieldMutatedProducer.setColumns(10);
		textFieldMutatedProducer.setBounds(529, 314, 89, 29);
		contentPane.add(textFieldMutatedProducer);

		chartMeanPanel = new Chart(Dictionary.getText("AVG_THIEFS"), Dictionary.getText("AVG_PRODUCERS"),
				Dictionary.getText("AVG_M_THIEFS"), Dictionary.getText("AVG_M_PRODUCERS"),
				populationAmount, Dictionary.getText("GRAPH_AVG_VAL"));
		chartMeanPanel.setBounds(0, 507, 650, 440);
		contentPane.add(chartMeanPanel);

		chartPanelThief = new Chart(Dictionary.getText("THIEFS"), Dictionary.getText("MUTATED_THIEFS"), Settings.getColor("THIEF_COLOR_SIMULATION"),
				 Settings.getColor("MUTATED_THIEF_COLOR_SIMULATION"), populationAmount, ViewConsts.THIEF_ICON48, Dictionary.getText("GRAPH_THIEFS"));
		chartPanelThief.setBounds(649, 507, 650, 440);
		contentPane.add(chartPanelThief);

		chartPanelProducer = new Chart(Dictionary.getText("PRODUCERS"), Dictionary.getText("MUTATED_PRODUCERS"),  Settings.getColor("PRODUCER_COLOR_SIMULATION"),
				 Settings.getColor("MUTATED_PRODUCER_COLOR_SIMULATION"), populationAmount, ViewConsts.PRODUCER_ICON48, Dictionary.getText("GRAPH_PRODUCERS"));
		chartPanelProducer.setBounds(649, 67, 650, 440);
		contentPane.add(chartPanelProducer);

		txtDelay = new JTextPane();
		txtDelay.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		txtDelay.setText(Dictionary.getText("DELAY"));
		txtDelay.setFont(new Font("Tahoma", Font.PLAIN, 14));
		txtDelay.setEditable(false);
		txtDelay.setBackground(Settings.getColor("FILEDS_WITH_TEXTS"));
		txtDelay.setBounds(525, 427, 93, 41);
		contentPane.add(txtDelay);

		delayTextField = new JTextField();
		delayTextField.setBorder(null);
		delayTextField.setBackground(Settings.getColor("FIELDS_WITH_NUMBERS"));
		delayTextField.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		delayTextField.setText("0");
		delayTextField.setHorizontalAlignment(SwingConstants.CENTER);
		delayTextField.setFont(new Font("Tahoma", Font.PLAIN, 12));
		delayTextField.setColumns(10);
		delayTextField.setBounds(525, 479, 54, 17);

		contentPane.add(delayTextField);

		lblMs = new JLabel(Dictionary.getText("ms"));
		lblMs.setBorder(null);
		lblMs.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		lblMs.setHorizontalAlignment(SwingConstants.CENTER);
		lblMs.setFont(new Font("Tahoma", Font.PLAIN, 12));
		lblMs.setBounds(583, 479, 35, 17);
		contentPane.add(lblMs);

		txtStartThiefsValue = new JTextField();
		txtStartThiefsValue.setBackground(Settings.getColor("FILEDS_WITH_TEXTS"));
		txtStartThiefsValue.setDisabledTextColor(Settings.getColor("SIMULATION_WINDOW_FONT"));
		txtStartThiefsValue.setEnabled(false);
		txtStartThiefsValue.setHorizontalAlignment(SwingConstants.CENTER);
		txtStartThiefsValue.setText(Dictionary.getText("START")+":" + thiefsAmount
				+ String.format("(%.1f)%%", (double) thiefsAmount / populationAmount * 100));
		txtStartThiefsValue.setBounds(20, 137, 116, 29);
		contentPane.add(txtStartThiefsValue);
		txtStartThiefsValue.setColumns(10);

		txtStartProducersValue = new JTextField();
		txtStartProducersValue.setBackground(Settings.getColor("FILEDS_WITH_TEXTS"));
		txtStartProducersValue.setDisabledTextColor(Settings.getColor("SIMULATION_WINDOW_FONT"));
		txtStartProducersValue.setEnabled(false);
		txtStartProducersValue.setHorizontalAlignment(SwingConstants.CENTER);
		txtStartProducersValue.setText(Dictionary.getText("START")+":" + producersAmount
				+ String.format("(%.1f)%%", 100d - (double) thiefsAmount / populationAmount * 100));
		txtStartProducersValue.setColumns(10);
		txtStartProducersValue.setBounds(20, 204, 116, 29);
		contentPane.add(txtStartProducersValue);

		thiefsMutationAmountInfo = new JTextPane();
		thiefsMutationAmountInfo.setDisabledTextColor(Settings.getColor("SIMULATION_WINDOW_FONT"));
		thiefsMutationAmountInfo.setEnabled(false);
		thiefsMutationAmountInfo.setForeground(UIManager.getColor("Label.foreground"));
		thiefsMutationAmountInfo.setBackground(Settings.getColor("FILEDS_WITH_TEXTS"));
		thiefsMutationAmountInfo.setFont(new Font("Tahoma", Font.PLAIN, 13));
		thiefsMutationAmountInfo.setText(Dictionary.getText("ART_INTRO") + " 0");
		thiefsMutationAmountInfo.setBounds(20, 273, 116, 30);
		contentPane.add(thiefsMutationAmountInfo);

		producersMutationAmountInfo = new JTextPane();
		producersMutationAmountInfo.setDisabledTextColor(Settings.getColor("SIMULATION_WINDOW_FONT"));
		producersMutationAmountInfo.setEnabled(false);
		producersMutationAmountInfo.setForeground(UIManager.getColor("Label.foreground"));
		producersMutationAmountInfo.setBackground(Settings.getColor("FILEDS_WITH_TEXTS"));
		producersMutationAmountInfo.setFont(new Font("Tahoma", Font.PLAIN, 13));
		producersMutationAmountInfo.setText(Dictionary.getText("ART_INTRO") + " 0");
		producersMutationAmountInfo.setBounds(20, 343, 116, 30);
		contentPane.add(producersMutationAmountInfo);

		btnReport = new JButton(Dictionary.getText("SHOW_REPORT"));
		btnReport.setToolTipText("<html><p width=\"100\">"+Dictionary.getText("RAPORT_BUTTON_INFO") + "</p></html>");
		btnReport.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		btnReport.setFont(new Font("Tahoma", Font.PLAIN, 15));
		btnReport.setEnabled(false);
		btnReport.setBackground(Settings.getColor("BUTTON_SHOW_REPORT_COLOR"));
		btnReport.setBounds(20, 467, 237, 29);
		contentPane.add(btnReport);

		populationPercentPane = new PopulationProportionPane();
		populationPercentPane.setBounds(1299, 109, 350, 771);
		contentPane.add(populationPercentPane);

		chartMeanPanel.setProportionPane(populationPercentPane);
		chartPanelThief.setProportionPane(populationPercentPane);
		chartPanelProducer.setProportionPane(populationPercentPane);

		width = new JSpinner();
		width.setBounds(1379, 922, 54, 25);

		width.setModel(new SpinnerNumberModel(populationPercentPane.getWidth() , 300 , 900 , 10));
		width.setBackground(Settings.getColor("FIELDS_WITH_NUMBERS"));
		width.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		width.setFont(new Font("Tahoma", Font.PLAIN, 10));
		contentPane.add(width);

		widthLbl = new JLabel(Dictionary.getText("WIDTH"));
		widthLbl.setHorizontalAlignment(SwingConstants.LEFT);
		widthLbl.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		widthLbl.setBounds(1309, 922, 64, 25);
		widthLbl.setToolTipText("300 - 900");
		contentPane.add(widthLbl);

		populationPercentPane.lockScroll(true);

		lblProportionGraph = new JLabel(Dictionary.getText("GRAPH_PROPORTION"));
		lblProportionGraph.setHorizontalAlignment(SwingConstants.CENTER);
		lblProportionGraph.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		lblProportionGraph.setFont(new Font("Tahoma", Font.BOLD, 30));
		lblProportionGraph.setBounds(1299, 68, 352, 36);
		contentPane.add(lblProportionGraph);

		lblBaseInfo = new JLabel(Dictionary.getText("BASE_INFORMATION_ABOUT_SIMULATION"));
		lblBaseInfo.setHorizontalAlignment(SwingConstants.CENTER);
		lblBaseInfo.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		lblBaseInfo.setFont(new Font("Tahoma", Font.BOLD, 30));
		lblBaseInfo.setBounds(10, 68, 640, 36);
		contentPane.add(lblBaseInfo);
		contentPane.add(btnMutate);

		lblLengthOfCycle = new JLabel(Dictionary.getText("LENGTH_OF_CYCLE_LBL") + " " + Dictionary.getText("LENGTH_OF_CYCLE_FROM").toLowerCase() + " " + getFrequencyFrom() + " " + Dictionary.getText("LENGTH_OF_CYCLE_TO").toLowerCase() + " " + getFrequencyTo() + " " + Dictionary.getText("LENGTH_OF_CYCLE_ROUND"));
		lblLengthOfCycle.setBounds(20, 384, 484, 32);
		lblLengthOfCycle.setHorizontalAlignment(SwingConstants.LEFT);
		lblLengthOfCycle.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		lblLengthOfCycle.setFont(new Font("Tahoma", Font.PLAIN, 20));
		contentPane.add(lblLengthOfCycle);

		rdbtnProportionMode = new JRadioButton(Dictionary.getText("PROPORTION_MODE"));
		rdbtnProportionMode.setBounds(1461, 887, 150, 23);
		rdbtnProportionMode.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		rdbtnProportionMode.setBackground(Settings.getColor("BACKGROUND_SIMULATION"));
		rdbtnProportionMode.setToolTipText("<html><p width=\"100\">"
				+ Dictionary.getText("PROPORTION_MODE_INFO")
				+ "</p></html>");
		contentPane.add(rdbtnProportionMode);

		rdbtnNormalMode = new JRadioButton(Dictionary.getText("NORMAL_MODE"));
		rdbtnNormalMode.setBounds(1309, 887, 150, 23);
		rdbtnNormalMode.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		rdbtnNormalMode.setBackground(Settings.getColor("BACKGROUND_SIMULATION"));
		rdbtnNormalMode.setToolTipText("<html><p width=\"100\">"
				+ Dictionary.getText("NORMAL_MODE_INFO")
				+ "</p></html>");
		rdbtnNormalMode.setSelected(true);
		contentPane.add(rdbtnNormalMode);

		modeChooser = new ButtonGroup();
	    modeChooser.add(rdbtnProportionMode);
	    modeChooser.add(rdbtnNormalMode);

		/**
		 * Uruchomienie symulacji
		 */
		countingThread = new Simulation(this);
	    
		/**
		 * Obsuga przeczania trybw panelu struktury populacji - tryb rankingowy.
		 */
	    rdbtnNormalMode.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if(rdbtnNormalMode.isSelected())
					populationPercentPane.setProportionMode(false);
				else
					populationPercentPane.setProportionMode(true);
			}
		});
	    
		/**
		 * Obsuga przeczania trybw panelu struktury populacji - tryb proporcjonalny.
		 */
	    rdbtnProportionMode.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if(rdbtnProportionMode.isSelected())
					populationPercentPane.setProportionMode(true);
				else
					populationPercentPane.setProportionMode(false);
			}
		});

		/**
		 * Obsuga przycisku start.
		 */
		btnStart.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				if (countingThread.isActive())
					countingThread.start();
				else
					countingThread.setActive(true);

				btnStart.setEnabled(false);
				btnReset.setEnabled(false);
				if (mutateThiefVsThief != 0.0 || mutateThiefVsProducer != 0.0 || mutateProducerVsProducer != 0.0
						|| mutateProducerVsThief != 0.0)
					btnMutate.setEnabled(true);
				btnStop.setEnabled(true);
				btnMutate.setEnabled(false);
				delayTextField.setEnabled(false);
				btnReport.setEnabled(false);
				width.setEnabled(false);
				countingThread.startCharting(chartMeanPanel, chartPanelThief, chartPanelProducer);

				populationPercentPane.lockScroll(true);
				populationPercentPane.setRun(true);
			}

		});

		/**
		 * Obsuga przycisku reset.
		 */
		btnReset.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{

				btnStart.setEnabled(true);
				btnStop.setEnabled(false);
				btnReport.setEnabled(false);
				btnMutate.setEnabled(true);
				btnReset.setEnabled(false);
				btnReport.setEnabled(false);
				width.setEnabled(true);

				countingThread.setActive(false);
				countingThread = new Simulation(SimulationWindow.this);
				chartMeanPanel.clear();
				chartPanelThief.clear();
				chartPanelProducer.clear();
				thiefsProgressBar.setValue(thiefsAmount);
				producersProgressBar.setValue(producersAmount);
				mutatedThiefsProgressBar.setValue(0);
				mutatedProducersProgressBar.setValue(0);
				thiefsMutationAmountInfo.setText(Dictionary.getText("ART_INTRO") + " 0");
				producersMutationAmountInfo.setText(Dictionary.getText("ART_INTRO") + " 0");
				textFieldThief.setText(thiefsAmount + "");
				textFieldProducer.setText(producersAmount + "");
				btnStop.setText(Dictionary.getText("STOP"));
				thiefMutate = false;
				producerMutate = false;

				clearHistoryOfPopulationStructure();

				populationPercentPane.lockScroll(true);
				populationPercentPane.setRun(false);

				if (mutateThiefVsThief != 0.0 || mutateThiefVsProducer != 0.0 || mutateProducerVsProducer != 0.0
						|| mutateProducerVsThief != 0.0)
					btnMutate.setEnabled(true);
				else
					btnMutate.setEnabled(false);

				repaint();
			}
		});

		/**
		 * Obsuga przycisku stop/restart.
		 */
		btnStop.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				btnReset.setEnabled(true);

				if(countingThread.isActive())
				{
					countingThread.setActive(false);
					btnStop.setText(Dictionary.getText("RESTART"));
					if (mutateThiefVsThief != 0.0 || mutateThiefVsProducer != 0.0 || mutateProducerVsProducer != 0.0
							|| mutateProducerVsThief != 0.0)
						btnMutate.setEnabled(true);
					delayTextField.setEnabled(true);
					btnReport.setEnabled(true);

					populationPercentPane.lockScroll(false);
					populationPercentPane.setRun(false);
					width.setEnabled(true);
				}
				else
				{
					countingThread.setActive(true);
					btnStop.setText(Dictionary.getText("STOP"));
					btnMutate.setEnabled(false);
					btnReport.setEnabled(false);
					btnReset.setEnabled(false);
					delayTextField.setEnabled(false);
					populationPercentPane.lockScroll(true);
					populationPercentPane.setRun(true);
					width.setEnabled(false);
				}
			}
		});

		/**
		 * Obsuga przycisku report.
		 */
		btnReport.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				try
				{
					ResultRaport raport = new ResultRaport();
					raport.setVisible(true);
					countingThread.raportToFrame(raport);
					raport.parseStrings();

				}
				catch (Exception ex)
				{
					ex.printStackTrace();
				}
			}
		});

		/**
		 * Obsuga przycisku mutuj.
		 */
		btnMutate.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(mutatedThiefsOn)
					thiefMutate = true;
				if(mutatedProducersOn)
					producerMutate = true;
				JOptionPane.showMessageDialog(null,
						Dictionary.getText("INTRODUCE_MUTATE_INFO"));
			}
		});

		/**
		 * Obsuga zamykania okna.
		 */
		this.addWindowListener(new java.awt.event.WindowAdapter()
		{
			@Override
			public void windowClosing(java.awt.event.WindowEvent windowEvent)
			{
				countingThread.flag = false;
			}
		});
		
		/**
		 * Obsuga focusw
		 */
		contentPane.addMouseListener(new MouseAdapter()
		{
			@Override
			public void mouseClicked(MouseEvent e)
			{
				super.mouseReleased(e);
				SimulationWindow.this.getContentPane().grabFocus();
			}
		});
		
		integerAmountWrapper(delayTextField, 0, 500);

		/**
		 * Obsuga zmiany rozmiaru okna i panelu struktury populacji.
		 */
		width.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e)
			{
				setSize(ViewConsts.SIMULATION_WINDOW_1 + (int) width.getValue(),  ViewConsts.SIMULATION_WINDOW_HEIGHT);
				populationPercentPane.setSize((int) width.getValue(), populationPercentPane.getHeight());
				lblProportionGraph.setSize((int) width.getValue(), lblProportionGraph.getHeight());
				lblTitle.setSize(1289 + (int) width.getValue(), lblTitle.getHeight());
			}
		});
	}
	public int getStrengthSelection() {
		return strengthSelection;
	}

	/**
	 * Obsuga wprowadzania danych w pola tekstowe.
	 * @param txtField
	 * @param minValue
	 * @param maxValue
	 */
	public void integerAmountWrapper(JTextField txtField, int minValue, int maxValue)
	{
		txtField.addKeyListener(new KeyAdapter()
		{
			public void keyTyped(KeyEvent e)
			{
				char c = e.getKeyChar();
				if (!((c >= '0') && (c <= '9') || (c == KeyEvent.VK_BACK_SPACE) || (c == KeyEvent.VK_DELETE)))
				{
					getToolkit().beep();
					e.consume();
				}
			}

			public void keyReleased(KeyEvent e)
			{

				if (!txtField.getText().isEmpty() && Integer.valueOf(txtField.getText()) >= maxValue)
				{
					txtField.setText(String.valueOf(maxValue));
				}
				if (txtField.getText().startsWith("0"))
					txtField.setText("0");
				txtField.postActionEvent();
			}

			public void keyPressed(KeyEvent e)
			{
				if (txtField.getText().length() >= 6)
				{
					e.consume();
					txtField.setText(String.valueOf(maxValue));
				}
			}

		});
		txtField.addFocusListener(new FocusAdapter()
		{
			public void focusGained(FocusEvent evt)
			{
				txtField.selectAll();
			}

			public void focusLost(FocusEvent e)
			{
				if (txtField.getText().isEmpty())
					txtField.setText(String.valueOf(minValue));
				if (txtField.getText().startsWith("0"))
					txtField.setText(String.valueOf(minValue));
				txtField.postActionEvent();
			}
		});
	}

	public void mutateThief(Double mutatethiefVsThief, Double mutatethiefVsProducer)
	{
		this.mutateThiefVsThief = mutatethiefVsThief;
		this.mutateThiefVsProducer = mutatethiefVsProducer;
		btnMutate.setEnabled(true);
	}

	public void mutateProducer(Double mutateProducerVsProducer, Double mutateProducerVsThief)
	{
		this.mutateProducerVsProducer = mutateProducerVsProducer;
		this.mutateProducerVsThief = mutateProducerVsThief;
		btnMutate.setEnabled(true);
	}

	public JProgressBar getThiefsProgressBar()
	{
		return thiefsProgressBar;
	}

	public void setThiefsProgressBar(JProgressBar thiefsProgressBar)
	{
		this.thiefsProgressBar = thiefsProgressBar;
	}

	public JProgressBar getProducersProgressBar()
	{
		return producersProgressBar;
	}

	public void setProducersProgressBar(JProgressBar producersProgressBar)
	{
		this.producersProgressBar = producersProgressBar;
	}

	public JProgressBar getMutatedThiefsProgressBar()
	{
		return mutatedThiefsProgressBar;
	}

	public void setMutatedThiefsProgressBar(JProgressBar mutatedThiefsProgressBar)
	{
		this.mutatedThiefsProgressBar = mutatedThiefsProgressBar;
	}

	public JProgressBar getMutatedProducersProgressBar()
	{
		return mutatedProducersProgressBar;
	}

	public void setMutatedProducersProgressBar(JProgressBar mutatedProducersProgressBar)
	{
		this.mutatedProducersProgressBar = mutatedProducersProgressBar;
	}

	public JTextField getTextFieldThief()
	{
		return textFieldThief;
	}

	public void setTextFieldThief(JTextField textFieldThief)
	{
		this.textFieldThief = textFieldThief;
	}

	public JTextField getTextFieldProducer()
	{
		return textFieldProducer;
	}

	public void setTextFieldProducer(JTextField textFieldProducer)
	{
		this.textFieldProducer = textFieldProducer;
	}

	public JTextField getTextFieldMutatedThief()
	{
		return textFieldMutatedThief;
	}

	public void setTextFieldMutatedThief(JTextField textFieldMutatedThief)
	{
		this.textFieldMutatedThief = textFieldMutatedThief;
	}

	public JTextField getTextFieldMutatedProducer()
	{
		return textFieldMutatedProducer;
	}

	public void setTextFieldMutatedProducer(JTextField textFieldMutatedProducer)
	{
		this.textFieldMutatedProducer = textFieldMutatedProducer;
	}

	public int getThiefsAmount()
	{
		return thiefsAmount;
	}

	public void setThiefsAmount(int thiefsAmount)
	{
		this.thiefsAmount = thiefsAmount;
	}

	public Double getThiefVsThief()
	{
		return thiefVsThief;
	}

	public void setThiefVsThief(Double thiefVsThief)
	{
		this.thiefVsThief = thiefVsThief;
	}

	public Double getProducerVsProducer()
	{
		return producerVsProducer;
	}

	public void setProducerVsProducer(Double producerVsProducer)
	{
		this.producerVsProducer = producerVsProducer;
	}

	public Double getThiefVsProducer()
	{
		return thiefVsProducer;
	}

	public void setThiefVsProducer(Double thiefVsProducer)
	{
		this.thiefVsProducer = thiefVsProducer;
	}

	public Double getProducerVsThief()
	{
		return producerVsThief;
	}

	public void setProducerVsThief(Double producerVsThief)
	{
		this.producerVsThief = producerVsThief;
	}

	public int getProducersAmount()
	{
		return producersAmount;
	}

	public void setProducersAmount(int producersAmount)
	{
		this.producersAmount = producersAmount;
	}

	public Double getMutateThiefVsThief()
	{
		return mutateThiefVsThief;
	}

	public void setMutatethiefVsThief(Double mutatethiefVsThief)
	{
		this.mutateThiefVsThief = mutatethiefVsThief;
	}

	public Double getMutateProducerVsProducer()
	{
		return mutateProducerVsProducer;
	}

	public void setMutateProducerVsProducer(Double mutateProducerVsProducer)
	{
		this.mutateProducerVsProducer = mutateProducerVsProducer;
	}

	public Double getMutateThiefVsProducer()
	{
		return mutateThiefVsProducer;
	}

	public void setMutateThiefVsProducer(Double mutateThiefVsProducer)
	{
		this.mutateThiefVsProducer = mutateThiefVsProducer;
	}

	public Double getMutateProducerVsThief()
	{
		return mutateProducerVsThief;
	}

	public void setMutateProducerVsThief(Double mutateProducerVsThief)
	{
		this.mutateProducerVsThief = mutateProducerVsThief;
	}

	public boolean isThiefMutate()
	{
		return thiefMutate;
	}

	public void setThiefMutate(boolean thiefMutate)
	{
		this.thiefMutate = thiefMutate;
	}

	public boolean isProducerMutate()
	{
		return producerMutate;
	}

	public void setProducerMutate(boolean producerMutate)
	{
		this.producerMutate = producerMutate;
	}
	
	public JTextField getDelayTextField()
	{
		return delayTextField;
	}

	public JTextField getTxtStartThiefsValue()
	{
		return txtStartThiefsValue;
	}

	public JTextField getTxtStartProducersValue()
	{
		return txtStartProducersValue;
	}

	public JTextPane getThiefMutationAmountInfo()
	{
		return thiefsMutationAmountInfo;
	}

	public JTextPane getProducerMutationAmountInfo()
	{
		return producersMutationAmountInfo;
	}

	public JPanel getContentPane()
	{
		return contentPane;
	}

	public int getFrequencyFrom() {
		return frequencyFrom;
	}

	public int getFrequencyTo() {
		return frequencyTo;
	}

	/**
	 * Wprowadz dane do historii struktury populacji.
	 * @param structer
	 * @param population
	 * @param howMuchSelect
	 */
	public void preparePopulationProportionPane(ProportionRow structer, ArrayList<PopulationMember> population, int howMuchSelect)
	{
		populationPercentPane.putProportionRow(structer);
		populationPercentPane.putSortedPopulation(population);
		populationPercentPane.setHowMuchSelect(howMuchSelect);
	}

	/**
	 * Wyczy chistorie w panelu struktury populacji.
	 */
	public void clearHistoryOfPopulationStructure()
	{
		populationPercentPane.clearList();
	}
}
